@@ -34,6 +34,7 @@
 #define INCLUDE_OPENPTS_H_
 
 #include <stdio.h>
+#include <unistd.h>
 
 #include <syslog.h>
 
@@ -56,6 +57,10 @@
 #include <openpts_fsm.h>
 #include <openpts_tpm.h>
 
+#ifdef CONFIG_TBOOT
+#include <openpts_tboot.h>
+#endif
+
 // TODO
 #ifdef AIX
 #define PPC
@@ -75,21 +80,20 @@ extern nl_catd catd;
 // #define gettext_noop(x) (x)
 #endif  // HAVE_CATGETS
 #else  // !ENABLE_NLS
+#ifdef NLS
+#undef NLS
+#endif
 #define NLS(a, b, x) x
 #endif  // ENABLE_NLS
 
 
 
 /* OpenPTS default configulations  */
-// port NUM
-//   http://www.iana.org/assignments/port-numbers
-//   http://www.iana.org/cgi-bin/usr-port-number.pl
-//   User ports [1024:49151]
-//     6674-6686  Unassigned
-// TODO 5556 is comfrict with Freeciv, => 6678
-// note) The port is local. for the remote access, we use SSH tunnel (port 22)
-#define PTSCD_PORT         6678
-#define PTSCD_CONFIG_FILE  "/etc/ptscd.conf"
+
+#define PTSCD_CONFIG_FILE  "/etc/ptsc.conf"
+#define PTSC_GROUP_NAME    "ptsc"
+#define PTSC_IFM_TIMEOUT   5
+#define PTSC_IFM_TIMEOUT_MAX   30
 
 #define MAXDATA 1024
 
@@ -190,6 +194,8 @@ typedef struct {
     int update_num; /**< num of update */
     void *update; /**< link to the last update */
 
+    int reset_pcr;  /**< resetPCR(n) in FSM */
+
     /* events */
     OPENPTS_PCR_EVENT_WRAPPER  *start; /**< */
     OPENPTS_PCR_EVENT_WRAPPER  *end;   /**< */
@@ -456,7 +462,8 @@ typedef struct {
     int autoupdate;  /**< 1:run autoupdate if selftest was failed at start */
 
     int srk_password_mode;
-    int tpm_resetdalock; /**< tpm.resetdalock=on */
+    int tpm_resetdalock; /**< tpm.resetdalock=on|off=1|0 */
+    int tpm_quote_type;  /**< tpm.quote.type=quote|quote2=1:0 */
 
     /* multiple manifest */
     OPENPTS_RMSETS *rmsets;
@@ -469,7 +476,9 @@ typedef struct {
     int   newrm_num;
     char *newrm_filename[MAX_RM_NUM];
 
-    char *ir_filename;
+    char *ir_dir;          /**< collector side */
+    char *ir_filename;     /**< vefirier side */
+
     char *prop_filename;
 
     int iml_endian;                /**< 0: same, 2:conv */
@@ -507,13 +516,11 @@ typedef struct {
     char *bios_vendor;
     char *bios_version;
 
-    /* collector(daemon) */
+    /* IF-M collector(ptsc) */
     char *hostname;
-    int   port;
-    /* collector(daemon) - ssh */
-    int   ssh_mode;
     char *ssh_username;
-    int   ssh_port;
+    char *ssh_port;
+    int   ifm_timeout;
 
 #ifdef CONFIG_AUTO_RM_UPDATE
     int enable_aru;            /**> Enable update scan */
@@ -556,6 +563,7 @@ typedef struct {
 
     /* TPM emu */
     OPENPTS_TPM_CONTEXT tpm; /**< */
+    int drtm;
 
     /* PCRs */
     int pcr_num;  // TODO(munetoh) move to pcrs->pcr_num
@@ -591,6 +599,7 @@ typedef struct {
     OPENPTS_IR_CONTEXT *ir_ctx;
 
 
+
     /* Runtime Validation */
     int bios_action_count;  // by snapshot
 #ifdef CONFIG_AIDE
@@ -603,12 +612,13 @@ typedef struct {
     int  ima_ignore;  // they are included in the valid count
 
 
-
     /* IF-M */
     BYTE *read_msg;
     OPENPTS_NONCE *nonce;
     PTS_UUID *uuid; /**< uuid of otherside, own uuid is ctx->conf->uuid */
     char *str_uuid;
+    UINT32 ifm_errno;  /**<  PTS error code */
+    char * ifm_strerror;
 
     /* misc ? */
     int cid; /**< */
@@ -622,8 +632,6 @@ typedef struct {
 /* conf.c */
 OPENPTS_CONFIG * newPtsConfig();
 int freePtsConfig(OPENPTS_CONFIG *conf);
-char *getFullpathName(char *base_path, char *filename);
-char *getFullpathDir(char *filename);
 int readPtsConfig(OPENPTS_CONFIG *conf, char *filename);
 int writeTargetConf(OPENPTS_CONFIG *conf, PTS_UUID *uuid, char *filename);
 int readTargetConf(OPENPTS_CONFIG *conf, char *filename);
@@ -643,10 +651,8 @@ int readFsmFromPropFile(OPENPTS_CONTEXT *ctx, char * filename);  // fsm.c -> ctx
 #define ALGTYPE_SHA1 0
 #define ALGTYPE_MD5  1
 
-
 /* ifm.c */
-// TODO refectoring
-int writePtsTlvToSock(OPENPTS_CONTEXT *ctx, int sock, int type);
+int writePtsTlv(OPENPTS_CONTEXT *ctx, int fd, int type);
 
 /* collector.c */
 int collector(
@@ -656,12 +662,14 @@ int collector(
 /* verifier.c */
 int verifier(
     OPENPTS_CONTEXT *ctx,
-    char * host, int port, char *conf_dir, int mode);
+    char *host, char *ssh_username, char *ssh_port, char *conf_dir, int mode);
 int enroll(
     OPENPTS_CONTEXT *ctx,
-    char * host, int port, char *conf_dir, int force);
+    char *host, char *ssh_username, char *ssh_port, char *conf_dir, int force);
 int writeAideIgnoreList(OPENPTS_CONTEXT *ctx, char *filename);
-int updateRm(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir);
+int updateRm(
+    OPENPTS_CONTEXT *ctx,
+    char *host, char *ssh_username, char *ssh_port, char *conf_dir);
 int extendEvCollectorStart(OPENPTS_CONFIG *conf);
 /* verifier mode */
 #define OPENPTS_VERIFY_MODE 0
@@ -760,7 +768,9 @@ int makeNewRmSetDir(OPENPTS_CONFIG *conf);
 /* ir.c */
 OPENPTS_IR_CONTEXT *newIrContext();
 void freeIrContext(OPENPTS_IR_CONTEXT *ctx);
+// TODO remove file
 int writeIr(OPENPTS_CONTEXT *ctx, const char *file);
+// int writeIr(OPENPTS_CONTEXT *ctx);
 int validateIr(OPENPTS_CONTEXT *ctx, const char *file);
 int genIr(OPENPTS_CONTEXT *ctx);
 
@@ -855,6 +865,10 @@ int saveToFile(char * filename, int len, BYTE * msg);
 int getUint32(BYTE *buf);
 int makeDir(char *dirname);
 int checkDir(char *dirname);
+ssize_t wrapRead(int fd, void *buf, size_t count);
+ssize_t wrapWrite(int fd, const void *buf, size_t count);
+char *getFullpathName(char *base_path, char *filename);
+char *getFullpathDir(char *filename);
 
 /* log.c */
 void writeLog(int priority, const char *format, ...);
@@ -894,6 +908,8 @@ int newrm(OPENPTS_CONFIG *conf, int prop_count, OPENPTS_PROPERTY *prop_start, OP
 #include "./openpts_aru.h"
 #endif
 
+/* ssh.c */
+pid_t ssh_connect(char *host, char *ssh_username, char *ssh_port, char *key_file, int *socket);
 
 /* print controll */
 
