@@ -26,7 +26,7 @@
  * \brief FSM action
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-06-19
- * cleanup 2011-01-22 SM
+ * cleanup 2011-04-26 SM
  *
  * FSM Action (UML2 doActivity)
  *
@@ -60,13 +60,14 @@ typedef struct {
     int  name_len;
     int  type;
     int (*func_1)(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper);
-    int (*func_2)(OPENPTS_CONTEXT *ctx, char* name, char* b64digest, char *integrity);  // TODO NA?
+    int (*func_2)(OPENPTS_CONTEXT *ctx, char *name, char* b64digest, char *integrity);  // TODO NA?
     int (*func_3)(OPENPTS_CONTEXT *ctx);
-    int (*func_4)(OPENPTS_CONTEXT *ctx, char * name);
+    int (*func_4)(OPENPTS_CONTEXT *ctx, char *name);
     // validateProperty
     int (*func_5)(OPENPTS_CONTEXT *ctx, char *name, char *value, char *action);
     // setProperty
     int (*func_6)(OPENPTS_CONTEXT *ctx, char *name, char *value, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper);
+    int (*func_7)(OPENPTS_CONTEXT *ctx, char *value, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper);
     int rc;
 } OPENPTS_ACTION_TABLE;
 
@@ -74,20 +75,42 @@ typedef struct {
 
 
 /**
- * reset PCR
+ *  resetPCR(pcr_index)
+ *
+ *  action   - reset PCR[pcr_index]
+ *  location - startup of the FSM
+ *
+ *  value    - string of pcr index value, 0-23
  */
-int resetPCR(OPENPTS_CONTEXT *ctx) {
-    // DEBUG("resetPCR - NA\n");
-    // TODO find best place - 2011-02-07 SM
+int resetPCR(OPENPTS_CONTEXT *ctx, char *value) {
+    int rc;
+    int pcr_index = -1;
+
+    pcr_index = atoi(value);
+
+    DEBUG_FSM("resetPCR(%d)\n", pcr_index);
+    rc = resetTpmPcr(&ctx->tpm, pcr_index);
+    if (rc != PTS_SUCCESS) {
+        ERROR("reset PCR[%d] was failed, check the model");
+        return PTS_INTERNAL_ERROR;
+    }
+
+    /* Also, reset the action counter */
     ctx->bios_action_count = 0;
+
     return PTS_SUCCESS;
 }
 
-
 /* BIOS */
 
 /**
- * addBIOSAction
+ * addBIOSAction()
+ *
+ *  EventData(string) => Properties
+ *
+ *  bios.pcr.N.action.C=eventdata[]
+ *    N: pcr_index
+ *    C: action counter
  */
 int addBIOSAction(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper) {
     TSS_PCR_EVENT *event;
@@ -108,18 +131,18 @@ int addBIOSAction(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper)
         return PTS_INTERNAL_ERROR;
     }
 
-    /* name = value */
+    /* value = eventdata */
     value = snmalloc((char *)event->rgbEvent, event->ulEventLength);
+    if (value == NULL) {
+        ERROR("no memory\n");
+        return PTS_INTERNAL_ERROR;
+    }
 
-#if 0
-    // event # -> action index
-    snprintf(name, BUF_SIZE, "bios.pcr.%d.action.%d", event->ulPcrIndex, eventWrapper->index);
-#else
-    // action index
-    // TODO reset ctx->bios_action_count at the start of snapshot
+    /* name */
+    // ctx->bios_action_count is reset by resetPCR()
     snprintf(name, BUF_SIZE, "bios.pcr.%d.action.%d", event->ulPcrIndex, ctx->bios_action_count);
     ctx->bios_action_count++;
-#endif
+
 
     DEBUG_FSM("[FSM] addBIOSAction() - '%s' = '%s'\n", name, value);
 
@@ -148,7 +171,6 @@ int addBIOSAction(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper)
  *
  */
 int addBIOSSpecificProperty(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper) {
-    int rc = -1;
     TSS_PCR_EVENT *event;
     UINT32 event_id;
     UINT32 event_length;
@@ -156,13 +178,13 @@ int addBIOSSpecificProperty(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eve
     /* event */
     if (eventWrapper == NULL) {
         ERROR("addBIOSSpecificProperty- eventWrapper is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1
     }
     event = eventWrapper->event;
 
     if (event->eventType != 0x06) {
         ERROR("addBIOSSpecificProperty - bad event type 0x%x !- 0x06\n", event->eventType);
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1
     }
 
     // DEBUG("event data size = %d\n", event->ulEventLength);
@@ -197,7 +219,7 @@ int addBIOSSpecificProperty(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eve
                 } else {
                     updateProperty(ctx, "bios.smbios", b64);
                 }
-                rc = 0;
+                // rc = 0;
                 free(b64);
             }
             break;
@@ -207,7 +229,7 @@ int addBIOSSpecificProperty(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eve
             break;
     }
 
-    return -1;
+    return PTS_SUCCESS;  // -1;
 }
 
 
@@ -215,21 +237,22 @@ int addBIOSSpecificProperty(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eve
 /* Grub */
 
 /**
- * validateMBR
+ * validateMBR()
+ *
  */
 int validateMBR(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper) {
     TSS_PCR_EVENT *event;
 
     if (eventWrapper == NULL) {
         ERROR("eventWrapper is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     event = eventWrapper->event;
 
     if (event == NULL) {
         ERROR("event is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     TODO("validateMBR - NA\n");
@@ -257,18 +280,18 @@ int validateEltoritoBootImage(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *e
 
     if (eventWrapper == NULL) {
         ERROR("eventWrapper is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     event = eventWrapper->event;
     if (event == NULL) {
         ERROR("event is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     updateProperty(ctx, "ipl.eltorito.integrity", "unknown");
 
-    return -1;
+    return PTS_SUCCESS;  // -1;
 }
 
 /**
@@ -291,14 +314,14 @@ int setModuleProperty(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrap
     /* check */
     if (eventWrapper == NULL) {
         ERROR("eventWrapper is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     event = eventWrapper->event;
 
     if (event == NULL) {
         ERROR("event is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     /* kernel.initrd.digest = PCR => B64 digest */
@@ -319,7 +342,7 @@ int setModuleProperty(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrap
         free(buf);
     }
 
-    return -1;
+    return PTS_SUCCESS;  // -1;
 }
 
 
@@ -356,14 +379,14 @@ int setLinuxKernelCmdlineAssertion(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPP
     /* input check */
     if (eventWrapper == NULL) {
         ERROR("eventWrapper is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     event = eventWrapper->event;
 
     if (event == NULL) {
         ERROR("event is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     /* copy(malloc) strings */
@@ -397,7 +420,7 @@ int setLinuxKernelCmdlineAssertion(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPP
 
     /* free */
     sfree(cmdline);
-    return 0;
+    return PTS_SUCCESS;
 }
 
 /* Grub */
@@ -410,7 +433,7 @@ int setLinuxKernelCmdlineAssertion(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPP
 int validateKernelCmdline(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper) {
     TODO("validateKernelCmdline - NA\n");
     updateProperty(ctx, "kernel.commandline", "TBD");
-    return -1;
+    return PTS_SUCCESS; // -1;
 }
 
 /* Linux - IMA */
@@ -436,14 +459,14 @@ int validateImaAggregate(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventW
     /* check */
     if (eventWrapper == NULL) {
         ERROR("eventWrapper is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     event = eventWrapper->event;
 
     if (event == NULL) {
         ERROR("event is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     /* init SHA1 */
@@ -466,33 +489,44 @@ int validateImaAggregate(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventW
     } else {
         /* MISS */
         updateProperty(ctx, "ima.aggregate", "invalid");
-#if 0
-        TODO("validateImaAggregate - Wrong IMA aggregete - check FSM, maybe it should use validateOldImaAggregate()\n");
-        {
+
+        if (verbose && DEBUG_FLAG) {
             int j;
-            printf("PCR   =");
+            BYTE pcr[SHA1_DIGEST_SIZE];
+            TODO("validateImaAggregate - "
+                 "Wrong IMA aggregete - check FSM, "
+                 "maybe it should use validateOldImaAggregate()\n");
+            printf("PCR   =  ");
             for (j = 0; j < (int) event->ulPcrValueLength; j ++) {
                 printf("%02x", event->rgbPcrValue[j]);
             }
             printf("\n");
 
-            printf("EDATA =");
+            for (i = 0; i < 8; i++) {
+                printf("PCR[%d] = ", i);
+                getTpmPcrValue(&ctx->tpm, i, pcr);
+                for (j = 0; j < SHA1_DIGEST_SIZE; j ++) {
+                    printf("%02x", pcr[j]);
+                }
+                printf("\n");
+            }
+
+            printf("EDATA  = ");
             for (j = 0; j < SHA1_DIGEST_SIZE; j ++) {
                 printf("%02x", event->rgbEvent[j]);
             }
-            printf("\n");
+            printf(" (extended value)\n");
 
-            printf("AGGREG=");
+            printf("AGGREG = ");
             for (j = 0; j < SHA1_DIGEST_SIZE; j ++) {
                 printf("%02x", digest[j]);
             }
-            printf("\n");
+            printf(" (cal value)\n");
         }
-#endif
     }
 
     // TODO(munetoh) also check the template and pcr?
-    return 0;
+    return PTS_SUCCESS;
 }
 
 /**
@@ -514,13 +548,13 @@ int validateOldImaAggregate(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eve
     /* check */
     if (eventWrapper == NULL) {
         ERROR("eventWrapper is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     event = eventWrapper->event;
     if (event == NULL) {
         ERROR("event is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     /* init SHA1 */
@@ -545,7 +579,7 @@ int validateOldImaAggregate(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eve
     }
     // TODO(munetoh) also check the eventdata string?
 
-    return 0;
+    return PTS_SUCCESS;
 }
 
 /**
@@ -569,7 +603,7 @@ int updateImaProperty(OPENPTS_CONTEXT *ctx, char* name, char* b64digest, char *i
     updateProperty(ctx, prop_name, b64digest);
 
     ctx->ima_count++;
-    return 0;
+    return PTS_SUCCESS;
 }
 
 
@@ -588,7 +622,9 @@ int updateImaProperty(OPENPTS_CONTEXT *ctx, char* name, char* b64digest, char *i
  *
  */
 int validateImaMeasurement(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper) {
+#ifdef CONFIG_AIDE
     TSS_PCR_EVENT *event;
+#endif
     // char b64digest[SHA1_BASE64_DIGEST_SIZE+1];
 
     DEBUG_CAL("validateImaMeasurement - start\n");
@@ -599,13 +635,14 @@ int validateImaMeasurement(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *even
         // DEBUG("validateImaMeasurement - eventWrapper is NULL\n");
         // DEBUG("validateImaMeasurement - eventWrapper is null\n");
         // return 1;  // =>  seg fault
-        return 0;  // => green
+        return PTS_SUCCESS;  // => green
         // TODO why?
     }
 
-    event = eventWrapper->event;
+
 
 #ifdef CONFIG_AIDE
+    event = eventWrapper->event;
     if (ctx->conf->ima_validation_mode == OPENPTS_VALIDATION_MODE_AIDE) {
         int rc = 0;
         char *name;
@@ -634,12 +671,12 @@ int validateImaMeasurement(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *even
 #endif
             eventWrapper->status = OPENPTS_RESULT_VALID;
             free(name);
-            return 0;
+            return PTS_SUCCESS;
         } else if (rc == 1) {
             // IGNORE
             eventWrapper->status = OPENPTS_RESULT_IGNORE;  // TODO
             free(name);
-            return 0;
+            return PTS_SUCCESS;
         } else if (rc == 2) {
             // MISS
             // DEBUG("validateImaMeasurement w/ AIDE - MISS name=[%s]\n", name);
@@ -657,13 +694,13 @@ int validateImaMeasurement(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *even
                 free(hex);
             }
             free(name);
-            return 0;
+            return PTS_SUCCESS;
         } else {
             // ERROR
             ERROR("validateImaMeasurement - checkEventByAide fail, rc - %d\n", rc);
             eventWrapper->status = PTS_INTERNAL_ERROR;  // OPENPTS_RESULT_INT_ERROR;
             free(name);
-            return -1;
+            return PTS_INTERNAL_ERROR;  // -1;
         }
         // TODO free md
         // freeAideMetadata(md);
@@ -677,11 +714,11 @@ int validateImaMeasurement(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *even
     }
 #endif
     else {
-        return 0;
+        return PTS_SUCCESS;
     }
 
     ERROR("validateImaMeasurement - ERROR\n");
-    return -1;
+    return PTS_INTERNAL_ERROR;  // -1;
 }
 
 /* IMA NG */
@@ -689,11 +726,11 @@ int validateImaMeasurement(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *even
 int validateImaAggregateNG(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper) {
     ERROR("validateImaAggregateNG - NA\n");
     updateProperty(ctx, "ima.aggregate", "TBD");
-    return -1;
+    return PTS_INTERNAL_ERROR;  // -1;
 }
 
 int validateImaMeasurementNG(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper) {
-    return -1;
+    return PTS_INTERNAL_ERROR;  // -1;
 }
 
 
@@ -821,7 +858,145 @@ int startCollector(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper
 }
 
 
+#ifdef CONFIG_TBOOT
+int addIntelTxtTbootProperty(OPENPTS_CONTEXT *ctx, OPENPTS_PCR_EVENT_WRAPPER *eventWrapper) {
+    TSS_PCR_EVENT *event;
 
+    DEBUG_FSM("addIntelTxtTbootProperty - start\n");
+
+    /* event */
+    if (eventWrapper == NULL) {
+        ERROR("addBIOSSpecificProperty- eventWrapper is NULL\n");
+        return -1;
+    }
+    event = eventWrapper->event;
+
+    switch (event->eventType) {
+        case EV_TBOOT_SINIT_V6:
+            {
+                OPENPTS_EVENT_TBOOT_SINIT_V6 *data;
+                char *buf;
+                data = (OPENPTS_EVENT_TBOOT_SINIT_V6 *) event->rgbEvent;
+                buf = getHexString(data->sinit_hash, 20);
+                updateProperty(ctx, "intel.txt.tboot.sinit.hash.hex", buf);
+                free(buf);
+                // TODO add rest
+            }
+            break;
+        case EV_TBOOT_SINIT_V7:
+            {
+                OPENPTS_EVENT_TBOOT_SINIT_V7 *data;
+                char *buf;
+                data = (OPENPTS_EVENT_TBOOT_SINIT_V7 *) event->rgbEvent;
+                buf = getHexString(data->sinit_hash, 32);
+                updateProperty(ctx, "intel.txt.tboot.sinit.hash.hex", buf);
+                free(buf);
+                // TODO add rest
+            }
+            break;
+        case EV_TBOOT_STM_V6:
+            {
+                OPENPTS_EVENT_TBOOT_STM_V6 *data;
+                char *buf;
+                data = (OPENPTS_EVENT_TBOOT_STM_V6 *) event->rgbEvent;
+                buf = getHexString(data->bios_acm_id, 20);
+                updateProperty(ctx, "intel.txt.tboot.bios.acm.id.hex", buf);
+                free(buf);
+                // TODO add rest
+            }
+            break;
+        case EV_TBOOT_POLCTL:
+            {
+                OPENPTS_EVENT_TBOOT_POLCTL *data;
+                char *buf;
+                data = (OPENPTS_EVENT_TBOOT_POLCTL *) event->rgbEvent;
+                buf = getHexString(data->pol_control, 4);
+                updateProperty(ctx, "intel.txt.tboot.pol.control.hex", buf);
+                buf = getHexString(data->pol_hash, 20);
+                updateProperty(ctx, "intel.txt.tboot.pol.hash.hex", buf);
+                free(buf);
+                // TODO add rest
+            }
+            break;
+        case EV_TBOOT_MLE_HASH:
+            {
+                char *buf;
+                buf = getHexString(event->rgbPcrValue, 20);
+                updateProperty(ctx, "intel.txt.tboot.mle.hash.hex", buf);
+                free(buf);
+            }
+            break;
+
+        case EV_TBOOT_MODULE:
+            {
+                OPENPTS_EVENT_TBOOT_MODULE *data;
+                char name[256];
+                char *value;
+                UINT32 size;
+                BYTE *ptr;
+
+                if (event->ulEventLength < 48) {
+                    // Bad EventData
+                    TODO("addIntelTxtTbootProperty() bad eventdata, size = %d\n",
+                        event->ulEventLength);
+                } else {
+                    // EventData
+                    data = (OPENPTS_EVENT_TBOOT_MODULE *) event->rgbEvent;
+
+                    snprintf(name, sizeof(name),
+                        "intel.txt.tboot.pcr.%d.module.command.hash.hex",
+                        event->ulPcrIndex);
+                    value = getHexString(data->command_hash, 20);
+                    updateProperty(ctx, name, value);
+                    free(value);
+
+                    snprintf(name, sizeof(name),
+                        "intel.txt.tboot.pcr.%d.module.file.hash.hex",
+                        event->ulPcrIndex);
+                    value = getHexString(data->file_hash, 20);
+                    updateProperty(ctx, name, value);
+                    free(value);
+
+                    snprintf(name, sizeof(name),
+                        "intel.txt.tboot.pcr.%d.module.command",
+                        event->ulPcrIndex);
+                    ptr = (BYTE *)&event->rgbEvent[40];
+                    size = *(UINT32*) ptr;
+                    ptr += 4;
+                    value = malloc(size + 1);
+                    memcpy(value, (BYTE *)ptr, size);
+                    value[size] = 0;
+                    updateProperty(ctx, name, value);
+                    free(value);
+
+                    snprintf(name, sizeof(name),
+                        "intel.txt.tboot.pcr.%d.module.filename",
+                        event->ulPcrIndex);
+                    ptr += size;
+                    size = *(UINT32*) ptr;
+                    ptr += 4;
+                    value = malloc(size + 1);
+                    memcpy(value, (BYTE *)ptr, size);
+                    value[size] = 0;
+                    updateProperty(ctx, name, value);
+                    free(value);
+                }
+            }
+            break;
+
+        default:
+            ERROR("Unknown event tupe 0x%x\n", event->eventType);
+            break;
+    }
+
+    /* set DRTM flag => resetPcr(1) at writeIr() */
+    // TODO
+    ctx->drtm = 1;
+
+    // updateProperty(ctx, "kernel.commandline", "TBD");
+    return PTS_SUCCESS;
+}
+#endif
 
 
 
@@ -852,10 +1027,10 @@ static OPENPTS_ACTION_TABLE action_table[] = {
         .type = 0,
         .rc = OPENPTS_FSM_FLASH
     }, {
-        .name = "resetPCR(",
-        .name_len = 9,
-        .type = 3,
-        .func_3 = resetPCR
+        .name = "resetPCR",
+        .name_len = 8,
+        .type = 4,
+        .func_4 = resetPCR
     }, {
         .name = "validateProperty(",
         .name_len = 17,
@@ -942,8 +1117,9 @@ static OPENPTS_ACTION_TABLE action_table[] = {
         .type = 4,
         .func_4 = saveCounter
     },
-    /* update function +5*/
+
 #ifdef CONFIG_AUTO_RM_UPDATE
+    /* update function 4 */
     {
         .name = "startUpdate",
         .name_len = 11,
@@ -965,6 +1141,15 @@ static OPENPTS_ACTION_TABLE action_table[] = {
         .type = 1,
         .func_1 = updateCollector
     },
+#endif
+#ifdef CONFIG_TBOOT
+    /* 1 */
+    {
+        .name = "addIntelTxtTbootProperty",
+        .name_len = 24,
+        .type = 1,
+        .func_1 = addIntelTxtTbootProperty
+    },
 #endif
     {
         .name = "startCollector",
@@ -974,7 +1159,19 @@ static OPENPTS_ACTION_TABLE action_table[] = {
     }
 };
 
-#define OPENPTS_ACTION_TABLE_SIZE (5 + 2 + 2 + 3 + 5 + 3 + 5)
+#ifdef CONFIG_AUTO_RM_UPDATE
+#define OPENPTS_ACTION_TABLE_ARU_SIZE  4
+#else
+#define OPENPTS_ACTION_TABLE_ARU_SIZE  0
+#endif
+
+#ifdef CONFIG_TBOOT
+#define OPENPTS_ACTION_TABLE_TBOOT_SIZE  1
+#else
+#define OPENPTS_ACTION_TABLE_TBOOT_SIZE  0
+#endif
+
+#define OPENPTS_ACTION_TABLE_SIZE (5 + 2 + 2 + 3 + 5 + 3 + OPENPTS_ACTION_TABLE_ARU_SIZE + OPENPTS_ACTION_TABLE_TBOOT_SIZE + 1)
 
 /**
  * doActivity
@@ -1005,12 +1202,12 @@ int doActivity(
     /* check */
     if (ctx == NULL) {
         ERROR("doActivity - ctx is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     if (action == NULL) {
         ERROR("doActivity - action is NULL\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;  // -1;
     }
 
     if (eventWrapper == NULL) {
@@ -1045,22 +1242,25 @@ int doActivity(
                 rc = action_table[i].rc;
                 goto end;
             case 1:
+                /* Action(Event) */
                 rc = action_table[i].func_1(ctx, eventWrapper);
                 goto end;
             case 2:
                 break;
             case 3:
+                /* Just call  Action() */
                 rc = action_table[i].func_3(ctx);
                 goto end;
             case 4:
-                /* name */
+                /* Action(Name) */
+                // INFO("doActivity type 4  %s", action);
                 name  = &buf[action_table[i].name_len + 1];  // 11 ; 1 =
                 name = strtok_r(name, ")", &saveptr);
                 name  = trim(name);
                 rc = action_table[i].func_4(ctx, name);
                 goto end;
             case 5:
-                /* name */
+                /* Action(Name,Value,Action) */
                 name  = &buf[action_table[i].name_len];
                 name = strtok_r(name, ", ", &saveptr);
                 value   = strtok_r(NULL, ")", &saveptr);
@@ -1071,7 +1271,7 @@ int doActivity(
                 rc = action_table[i].func_5(ctx, name, value, action);  // validateProperty
                 goto end;
             case 6:
-                /* name */
+                /* Action(Name,Value,Event) */
                 name  = &buf[action_table[i].name_len];
                 name = strtok_r(name, ", ", &saveptr);
                 value   = strtok_r(NULL, ")", &saveptr);
@@ -1081,6 +1281,13 @@ int doActivity(
 
                 rc = action_table[i].func_6(ctx, name, value, eventWrapper);  // setProperty
                 goto end;
+            case 7:
+                /* Action(Value,Event) */
+                value  = &buf[action_table[i].name_len + 1];  // 11 ; 1 =
+                value = strtok_r(value, ")", &saveptr);
+                value  = trim(value);
+                rc = action_table[i].func_7(ctx, value, eventWrapper);
+                goto end;
             default:
                 ERROR("unknown OPENPTS_ACTION_TABLE func tyoe\n");
                 break;
