@@ -1319,7 +1319,8 @@ int convertAideDbfileToSQLiteDbFile(char * aide_filename, char * sqlite_filename
     }
 
     sqlite3_exec(db,
-        "CREATE TABLE sample (id INTEGER PRIMARY KEY, digest TEXT NOT NULL, name TEXT NOT NULL, state INTEGER NOT NULL)",
+        "CREATE TABLE sample (id INTEGER PRIMARY KEY, digest TEXT NOT NULL, "
+        "name TEXT NOT NULL, state INTEGER NOT NULL)",
         NULL, NULL, &err);
     // DEBUG("CREATE err=%s\n", err);
 
@@ -1332,7 +1333,8 @@ int convertAideDbfileToSQLiteDbFile(char * aide_filename, char * sqlite_filename
     j = 0;
     for (i = 0; i < ctx->metadata_num; i++) {
         if (md->hash_key != NULL) {
-            sql = sqlite3_mprintf("INSERT INTO sample (id, digest, name, state)  VALUES (%d, '%s','%s', %d)",
+            sql = sqlite3_mprintf(
+                "INSERT INTO sample (id, digest, name, state)  VALUES (%d, '%s','%s', %d)",
                     j, md->hash_key, md->name, 0);
             sqlite3_exec(db, sql, NULL, NULL, &err);
             // DEBUG("INSERT err=%s\n", err);
