@@ -26,7 +26,7 @@
  * \brief TCG IF-M collector functions
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2011-01-06
- * cleanup 2011-01-21 SM
+ * cleanup 2011-04-26 SM
  *
  * move from ptscd.c 
  *
@@ -112,7 +112,7 @@ int extendEvCollectorStart(OPENPTS_CONFIG *conf) {
 
 
     /* get PCR value*/
-    // memcpy(&collector_start->pcr_value;
+    // memcpy(&collector_start->pcr_value;make
     readPcr(conf->openpts_pcr_index, pcr);
     memcpy(&collector_start->pcr_value, pcr, SHA1_DIGEST_SIZE);
 
@@ -147,7 +147,7 @@ int extendEvCollectorStart(OPENPTS_CONFIG *conf) {
 
 
 /**
- * initialize ptscd
+ * initialize ptsc
  *
  * 1. generate UUID
  * 2. generate Sign Key (NA)
@@ -155,7 +155,7 @@ int extendEvCollectorStart(OPENPTS_CONFIG *conf) {
  * 4. generate RM
  * 
  *
- * ./src/ptscd -i -c tests/data/Fedora12/ptscd.conf
+ * ./src/ptsc -i -c tests/data/Fedora12/ptscd.conf
  *
  * Return
  *  PTS_SUCCESS
@@ -228,7 +228,9 @@ int init(
     if (keygen == 1) {
         rc = createTssSignKey(conf->uuid->uuid, ps_type, NULL, 0, conf->srk_password_mode);
         if (rc == 0x0001) {  // 0x0001
-            fprintf(stderr, "createSignKey failed. if you uses well known SRK secret, all zeros (20 bytes of zeros) try -z option\n");
+            fprintf(stderr, "createSignKey failed. "
+                            "if you uses well known SRK secret, "
+                            "all zeros (20 bytes of zeros) try -z option\n");
             rc = PTS_INTERNAL_ERROR;
             goto free;
         } else if (rc != PTS_SUCCESS) {
@@ -366,7 +368,7 @@ int init(
             ERROR("missing RM file for level %d\n", i);
         }
     }
-    printf("\nptscd is successfully initialized!\n");
+    printf("\nptsc is successfully initialized!\n");
 
  free:
     /* free */
@@ -434,9 +436,11 @@ int selftest(OPENPTS_CONFIG *conf, int prop_count, OPENPTS_PROPERTY *prop_start,
     ctx->nonce->nonce_length = 20;
     ctx->nonce->nonce = malloc(20);
     memset(ctx->nonce->nonce, 0x5A, 20);
+    // dummy target uuid
+    ctx->str_uuid = smalloc("SELFTEST");
 
     /* gen IR */
-    rc = genIr(ctx);
+    rc = genIr(ctx);  // ir.c
     if (rc != PTS_SUCCESS) {
         ERROR("selftest() - genIR failed\n");
         rc = PTS_INTERNAL_ERROR;
@@ -499,7 +503,9 @@ int selftest(OPENPTS_CONFIG *conf, int prop_count, OPENPTS_PROPERTY *prop_start,
     // ctx->conf->iml_mode = 1;
     // ctx->conf->ir_without_quote = 1;
 
-    result = validateIr(ctx, ctx->conf->ir_filename);  /* ir.c */
+
+
+    result = validateIr(ctx, conf->ir_filename);  /* ir.c */
 
 
 
@@ -571,7 +577,7 @@ int selftest(OPENPTS_CONFIG *conf, int prop_count, OPENPTS_PROPERTY *prop_start,
  * 4. generate RM
  * 
  *
- * ./src/ptscd -i -c tests/data/Fedora12/ptscd.conf
+ * ./src/ptsc -i -c tests/data/Fedora12/ptscd.conf
  *
  * Return
  *  PTS_SUCCESS
@@ -699,7 +705,7 @@ int newrm(OPENPTS_CONFIG *conf, int prop_count, OPENPTS_PROPERTY *prop_start, OP
             ERROR("missing RM file for level %d\n", i);
         }
     }
-    // printf("\nptscd is successfully initialized!\n");
+    // printf("\nptsc is successfully initialized!\n");
 
  free:
     /* free */
@@ -710,7 +716,7 @@ int newrm(OPENPTS_CONFIG *conf, int prop_count, OPENPTS_PROPERTY *prop_start, OP
 
 
 /**
- * Print the configulation of PTS collector
+ * Print the configuration of PTS collector
  *
  * Return
  *   PTS_SUCCESS
@@ -724,7 +730,6 @@ int printCollectorStatus(OPENPTS_CONFIG *conf) {
     printf("%s version %s \n\n", PACKAGE, VERSION);
 
     printf("config file                 : %s\n", conf->config_file);
-    printf("poer                        : %d\n", conf->port);
     /* UUID */
     printf("UUID                        : %s (%s)\n", ctx->conf->uuid->str, conf->uuid->filename);
 
