@@ -23,11 +23,11 @@
 
 /**
  * \file src/conf.c
- * \brief read/write configulation file
+ * \brief read/write configuration file
  *
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-08-13
- * cleanup 2011-01-21 SM
+ * cleanup 2011-04-26 SM
  *
  *
  * grep strncmp src/conf.c | gawk '{print $3}'
@@ -46,10 +46,12 @@
  *  config.dir
  *  config.dir
  *  hostname
+ *  ifm.timeout
  *  ima.validation.mode
  *  iml.aligned
  *  iml.endian
  *  iml.mode
+ *  ir.dir
  *  ir.file
  *  ir.quote
  *  little
@@ -81,7 +83,6 @@
  *  srk.password.mode        null/known
  *  ------------------------------------------------------------------------
  *  
- *  
  */
 
 #include <stdio.h>
@@ -170,6 +171,8 @@ OPENPTS_CONFIG * newPtsConfig() {
     // set by configure.in
     conf->openpts_pcr_index = OPENPTS_PCR_INDEX;
 
+    conf->ifm_timeout = PTSC_IFM_TIMEOUT;
+
     return conf;
 }
 
@@ -205,6 +208,10 @@ int freePtsConfig(OPENPTS_CONFIG * conf) {
     if (conf->ir_filename != NULL) {
         free(conf->ir_filename);
     }
+    if (conf->ir_dir != NULL) {
+        free(conf->ir_dir);
+    }
+
     if (conf->prop_filename != NULL) {
         free(conf->prop_filename);
     }
@@ -298,6 +305,9 @@ int freePtsConfig(OPENPTS_CONFIG * conf) {
     if (conf->ssh_username != NULL) {
         free(conf->ssh_username);
     }
+    if (conf->ssh_port != NULL) {
+        free(conf->ssh_port);
+    }
 
     if (conf->config_file != NULL) {
         // DEBUG("conf->config_file => free\n");
@@ -310,141 +320,7 @@ int freePtsConfig(OPENPTS_CONFIG * conf) {
 }
 
 
-/**
- * get fullpathname of file
- * This malloc new buf for the fullpathname. 
- * In config
- *
- * basepath must be start from /
- *  
- * UnitTest : check_conf
- *
- */
-char *getFullpathName(char *basepath, char *filename) {
-    char *fullpath = NULL;
-    int basepath_len;
-    int filename_len;
-    int slash = 0;
 
-    /* check */
-    if (filename == NULL) {
-        ERROR("getFullpathName - filename is NULL\n");
-        return NULL;
-    }
-
-    /* start from root */
-    if (filename[0] == '/') {
-        /* seems fullpath, copy the filename to new buf */
-        fullpath = smalloc(filename);
-        return fullpath;
-    }
-
-    /* basepath + filename */
-    if (basepath == NULL) {
-        ERROR("getFullpathName - basepath is NULL, filename is %s\n", filename);
-        return NULL;
-    }
-
-
-    if (basepath[0] != '/') {
-        /* relative path -> error when it run as daemon */
-        TODO("getFullpathName() - basepath, '%s' is not started from root\n", basepath);
-    }
-
-
-    /*
-      rule
-
-        0x00 /AAA/ +   BBB => /AAA/BBB
-        0x01 /AAA/ + ./BBB => /AAA/BBB
-        0x10 /AAA  +   BBB => /AAA/BBB
-        0x11 /AAA  + ./BBB => /AAA/BBB  
-    */
-    basepath_len = strlen(basepath);
-    filename_len = strlen(filename);
-
-    if (filename_len < 2) {
-        ERROR("ilename len < 2\n");
-        return NULL;
-    }
-
-    /* basepath has "/" at end. else add "/" */
-    if (basepath[basepath_len - 1] !=  '/') {
-        slash = 0x10;
-    }
-    /* filename has "./" at start ? remove */
-    if ((filename[0] ==  '.') && (filename[1] ==  '/')) {
-        slash |= 0x01;
-    }
-
-    /* */
-    switch (slash) {
-        case 0x00:
-            /* /AAA/ +   BBB => /AAA/BBB */
-            fullpath = malloc(basepath_len + filename_len + 1);
-            memcpy(fullpath, basepath, basepath_len);
-            memcpy(&fullpath[basepath_len], filename, filename_len);
-            fullpath[basepath_len + filename_len] = 0;
-            break;
-        case 0x01:
-            /* /AAA/ + ./BBB => /AAA/BBB */
-            fullpath = malloc(basepath_len + filename_len + 1 - 2);
-            memcpy(fullpath, basepath, basepath_len);
-            memcpy(&fullpath[basepath_len], filename + 2, filename_len - 2);
-            fullpath[basepath_len + filename_len - 2] = 0;
-            break;
-        case 0x10:
-            /* /AAA  +   BBB => /AAA/BBB */
-            fullpath = malloc(basepath_len + 1 + filename_len + 1);
-            memcpy(fullpath, basepath, basepath_len);
-            fullpath[basepath_len] = '/';
-            memcpy(&fullpath[basepath_len + 1], filename, filename_len);
-            fullpath[basepath_len + filename_len + 1] = 0;
-            break;
-        case 0x11:
-            /* /AAA  + ./BBB => /AAA/BBB */
-            fullpath = malloc(basepath_len + 1 + filename_len + 1 - 2);
-            memcpy(fullpath, basepath, basepath_len);
-            fullpath[basepath_len] = '/';
-            memcpy(&fullpath[basepath_len + 1], filename + 2, filename_len - 2);
-            fullpath[basepath_len + filename_len - 1] = 0;
-            break;
-        default:
-            ERROR("internal error\n");
-            break;
-    }  // switch
-
-    return fullpath;
-}
-
-/**
- * Get dirname from fullpath filename
- * 
- * this malloc new string
- *
- * /AAA/BBB/CCC/DDD => /AAA/BBB/CCC/ 
- *
- */
-char *getFullpathDir(char *filename) {
-    char *fullpath = NULL;
-    char *slash;
-    int filename_len;
-    int i;
-
-    filename_len = strlen(filename);
-
-    for (i = filename_len; i > 0; i--) {
-        if (filename[i] == '/') {
-            slash = &filename[i];
-            break;
-        }
-    }
-
-    fullpath = malloc(i+2);
-    memcpy(fullpath, filename, i+1);
-    fullpath[i+1] = 0;
-    return fullpath;
-}
 
 
 /**
@@ -513,8 +389,6 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
     }
 
 
-    conf->port = PTSCD_PORT;  // set default
-
     /* open */
 
     if ((fp = fopen(filename2, "r")) == NULL) {
@@ -577,7 +451,8 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     conf->srk_password_mode = 1;
                     DEBUG("conf->srk_password_mode    : known\n");
                 } else {
-                    ERROR("Bad srk.password.mode flag %s\n", value);
+                    ERROR("Bad srk.password.mode flag '%s' in %s\n",
+                        value, filename);
                 }
             }
 
@@ -587,13 +462,26 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                     conf->tpm_resetdalock = 1;
                     DEBUG("conf->tpm_resetdalock      : on\n");
                 } else if (!strncmp(value, "off", 3)) {
-                    conf->tpm_resetdalock = 0;
+                    conf->tpm_resetdalock = 0;  // default
                     DEBUG("conf->tpm_resetdalock      : off (default)\n");
                 } else {
-                    ERROR("Bad tpm.resetdalock flag %s\n", value);
+                    ERROR("Bad tpm.resetdalock flag '%s' in %s\n",
+                        value, filename);
                 }
             }
 
+            /* tpm.quote.type */
+            if (!strncmp(name, "tpm.quote.type", 14)) {
+                if (!strncmp(value, "quote2", 6)) {
+                    conf->tpm_quote_type = 0;  // default
+                    DEBUG("conf->tpm_quote_type       : quote2 (default)\n");
+                } else if (!strncmp(value, "quote", 5)) {
+                    conf->tpm_quote_type = 1;
+                    DEBUG("conf->tpm_quote_type       : quote\n");
+                } else {
+                    ERROR("Bad tpm.quote.type flag %s\n", value);
+                }
+            }
 
             /* Endian for cross platform debug */
             if (!strncmp(name, "iml.endian", 10)) {
@@ -666,7 +554,7 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                 DEBUG("conf->rm_num               : %d\n", conf->rm_num);
             }
 
-            /* IR */
+            /* IR file (verifier side) */
             if (!strncmp(name, "ir.file", 7)) {
                 if (conf->ir_filename != NULL) {
                     // DEBUG("realloc conf->ir_filename");  // TODO realloc happen
@@ -675,6 +563,15 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                 conf->ir_filename = getFullpathName(config_path, value);
                 DEBUG("conf->ir_filename          : %s\n", conf->ir_filename);
             }
+            /* IR dir (collector side) */
+            if (!strncmp(name, "ir.dir", 6)) {
+                if (conf->ir_dir != NULL) {
+                    // DEBUG("realloc conf->ir_filename");  // TODO realloc happen
+                    free(conf->ir_dir);
+                }
+                conf->ir_dir = getFullpathName(config_path, value);
+                DEBUG("conf->ir_filename          : %s\n", conf->ir_dir);
+            }
 
             if (!strncmp(name, "prop.file", 9)) {
                 if (conf->prop_filename != NULL) {
@@ -902,29 +799,20 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
             }
 
             /* SSH */
-            if (!strncmp(name, "ssh.mode", 8)) {
-                if (!strncmp(value, "on", 2)) {
-                    conf->ssh_mode = OPENPTS_SSH_MODE_ON;
-                } else if (!strncmp(value, "off", 3)) {
-                    conf->ssh_mode = OPENPTS_SSH_MODE_OFF;
-                } else {
-                    ERROR("unknown ssh.mode %s\n", value);  // TODO
-                }
-            }
+            /*     default values */
+            conf->ssh_username = NULL;  // use default values
+            conf->ssh_port = NULL;
+
             if (!strncmp(name, "ssh.username", 12)) {
-               conf->ssh_username = smalloc(value);
-                DEBUG("conf->ssh_username         : %d\n", conf->ssh_username);
+                conf->ssh_username = smalloc(value);
+                DEBUG("conf->ssh_username         : %s\n", conf->ssh_username);
             }
             if (!strncmp(name, "ssh.port", 8)) {
-                conf->ssh_port = atoi(value);
-                DEBUG("conf->ssh_port             : %d\n", conf->ssh_port);
+                conf->ssh_port = smalloc(value);
+                DEBUG("conf->ssh_port             : %s\n", conf->ssh_port);
             }
 
-            /* port */
-            if (!strncmp(name, "port", 4)) {
-                conf->port = atoi(value);
-                DEBUG("conf->port                 : %d\n", conf->port);
-            }
+            /* hostname */
             if (!strncmp(name, "hostname", 8)) {
                 if (conf->hostname != NULL) {
                     // DEBUG("realloc conf->hostname\n");  // TODO realloc happen
@@ -934,6 +822,19 @@ int readPtsConfig(OPENPTS_CONFIG *conf, char *filename) {
                 DEBUG("conf->hostname             : %s\n", conf->hostname);
             }
 
+            /* IF-M timeout  */
+            if (!strncmp(name, "ifm.timeout", 11)) {
+                conf->ifm_timeout = atoi(value);
+                if (conf->ifm_timeout > PTSC_IFM_TIMEOUT_MAX) {
+                    DEBUG("conf->ifm_timeout          : %d => %d(MAX)\n",
+                        conf->ifm_timeout, PTSC_IFM_TIMEOUT_MAX);
+                    conf->ifm_timeout = PTSC_IFM_TIMEOUT_MAX;
+                } else {
+                    DEBUG("conf->ifm_timeout          : %d\n",
+                        conf->ifm_timeout);
+                }
+            }
+
             /* Selftest */
             if (!strncmp(name, "selftest", 8)) {
                 if (!strncmp(value, "on", 2)) {
@@ -1047,24 +948,22 @@ int writeTargetConf(OPENPTS_CONFIG *conf, PTS_UUID *uuid, char *filename) {
        fprintf(fp, "ima.validation.mode=none\n");
     }
 
-// 2011-03-04 SM 
+// 2011-03-04 SM
 // #ifdef CONFIG_AUTO_RM_UPDATE
 //     fprintf(fp, "autoupdate=on\n");
 // #endif
 
     /* SSH */
-    if (conf->ssh_mode == OPENPTS_SSH_MODE_ON) {
-       fprintf(fp, "ssh.mode=on\n");
-       fprintf(fp, "ssh.username=%s\n", conf->ssh_username);
-       fprintf(fp, "ssh.port=%d\n", conf->ssh_port);
-    } else {
-       fprintf(fp, "ssh.mode=off\n");
+    if (conf->ssh_username != NULL) {
+        fprintf(fp, "ssh.username=%s\n", conf->ssh_username);
+    }
+    if (conf->ssh_port != NULL) {
+        fprintf(fp, "ssh.port=%s\n", conf->ssh_port);
     }
 
     /* target hostname */
     // 20110117 move from dir name to conf, since the dir name uses UUID
     fprintf(fp, "hostname=%s\n", conf->hostname);
-    fprintf(fp, "port=%d\n", conf->port);
 
     fclose(fp);
     free(str_uuid);
@@ -1084,7 +983,6 @@ int readTargetConf(OPENPTS_CONFIG *conf, char *filename) {
 
     /* misc */
     conf->iml_mode = 0;  // set TSS
-    conf->ssh_mode = 0;  // local
     conf->rm_num = 0;
 
     rc = readPtsConfig(conf, filename);
