@@ -742,7 +742,7 @@ int addFsmTransition(
  * get Event String (malloc)
  */
 char *getEventString(OPENPTS_PCR_EVENT_WRAPPER *eventWrapper) {
-    int len;
+    // int len;
     int size = FSM_BUF_SIZE;  // TODO fixed size
     TSS_PCR_EVENT *event;
     char *buf;
@@ -757,7 +757,7 @@ char *getEventString(OPENPTS_PCR_EVENT_WRAPPER *eventWrapper) {
     /* event */
     event = eventWrapper->event;
     if (event != NULL) {
-        len = snprintf(buf, size, "PCR[%d],TYPE=%d", (int)event->ulPcrIndex, event->eventType);
+        // len = snprintf(buf, size, "PCR[%d],TYPE=%d", (int)event->ulPcrIndex, event->eventType);
     } else {
         ERROR("NULL event\n");  // TODO(munetoh)
         free(buf);
