@@ -217,7 +217,7 @@ int addEventToSnapshotBhv(
             /* level 0 SS is null => check Level 1 SS */
             ss = getSnapshotFromTable(ctx->ss_table, index, 1);
             if (ss == NULL) {
-                addReason(ctx, "[PCR%02d] Snapshot(FSM) is missing for PCR%d. Please check the configulation file '%s'",
+                addReason(ctx, "[PCR%02d] Snapshot(FSM) is missing for PCR%d. Please check the configuration file '%s'",
                     index,
                     index, ctx->conf->config_file);
                 ctx->ss_table->error[index] = PTS_INTERNAL_ERROR;
@@ -234,7 +234,7 @@ int addEventToSnapshotBhv(
             } else {
                 /* FSM is missing */
                 addReason(ctx,
-                    "[RM01-PCR%02d] FSM is missing for PCR%d, Level 1. Please check the configulation file '%s'",
+                    "[RM01-PCR%02d] FSM is missing for PCR%d, Level 1. Please check the configuration file '%s'",
                     index,
                     index, ctx->conf->config_file);
                 ctx->ss_table->error[index] = PTS_INTERNAL_ERROR;
@@ -252,7 +252,7 @@ int addEventToSnapshotBhv(
             if (ss == NULL) {
                 /* SS is missing */
                 addReason(ctx,
-                    "[PCR%02d] Snapshot is missing for PCR%d for Level 0 and 1. Please check the configulation file '%s'",
+                    "[PCR%02d] Snapshot is missing for PCR%d for Level 0 and 1. Please check the configuration file '%s'",
                     index,
                     index,
                     ctx->conf->config_file);
@@ -269,7 +269,7 @@ int addEventToSnapshotBhv(
             } else {
                 /* FSM is missing*/
                 addReason(ctx,
-                    "[RM01-PCR%02d] FSM is missing for PCR%d, Level 1. Please check the configulation file '%s'",
+                    "[RM01-PCR%02d] FSM is missing for PCR%d, Level 1. Please check the configuration file '%s'",
                     index,
                     index, ctx->conf->config_file);
                 ctx->ss_table->error[index] = PTS_INTERNAL_ERROR;
@@ -283,7 +283,7 @@ int addEventToSnapshotBhv(
             /* SS is missing */
             DEBUG("ss == NULL  =>  Reason\n");
             addReason(ctx,
-                "[RM%02d-PCR%02d] Snapshot is missing for PCR%d, Level %d. Please check the configulation file '%s'",
+                "[RM%02d-PCR%02d] Snapshot is missing for PCR%d, Level %d. Please check the configuration file '%s'",
                 active_level,
                 index,
                 index,
@@ -297,7 +297,7 @@ int addEventToSnapshotBhv(
             /* FSm is missing */
             DEBUG("ss->fsm_behavior == NULL  =>  Reason\n");
             addReason(ctx,
-                "[RM%02d-PCR%02d] FSM is missing for PCR%d, Level %d. Please check the configulation file '%s'",
+                "[RM%02d-PCR%02d] FSM is missing for PCR%d, Level %d. Please check the configuration file '%s'",
                 active_level,
                 index,
                 active_level,
@@ -570,8 +570,17 @@ int flashSnapshot(
     /* Get Snapshot */
     ss = getSnapshotFromTable(ctx->ss_table, index, active_level);
     if (ss == NULL) {
-        ERROR("No Snapshot at level %d\n", active_level);
-        return PTS_INTERNAL_ERROR;
+        ERROR("No Snapshot at PCR[%d]. level %d\n", index, active_level);
+        // return PTS_INTERNAL_ERROR;
+        // TODO 2011-05-02
+        active_level++;
+        ss = getSnapshotFromTable(ctx->ss_table, index, active_level);
+        if (ss == NULL) {
+            ERROR("No Snapshot at PCR[%d], level %d\n", index, active_level);
+            return PTS_INTERNAL_ERROR;
+        } else {
+            DEBUG("Skip Null SS level. level = %d\n", active_level);
+        }
     }
 
     if (active_level == 0) {
@@ -682,14 +691,14 @@ int getIml(OPENPTS_CONTEXT * ctx, int option) {
     TSS_PCR_EVENT *pcrEvents;
     UINT32 ulEventNumber = 0;
     OPENPTS_PCR_EVENT_WRAPPER *ew_new = NULL;
-    OPENPTS_PCR_EVENT_WRAPPER *ew_last = NULL;
+    // OPENPTS_PCR_EVENT_WRAPPER *ew_last = NULL;
     int i;
     int error = 0;
 
     DEBUG_CAL("getIml - start\n");
 
     /* clean up TPM */
-    resetTpm(&ctx->tpm);  // reset TPM
+    resetTpm(&ctx->tpm, 0);  // reset TPM DRTM=off
 
     /* check SS table */
     if (ctx->ss_table == NULL) {
@@ -733,7 +742,7 @@ int getIml(OPENPTS_CONTEXT * ctx, int option) {
     if (option == 0) {  // simple, snapshot[i] hold all events on PCR[i]
         TSS_PCR_EVENT *tpe_tss;
         TSS_PCR_EVENT *tpe;
-        int index;
+        // int index;
 
         for (i = 0; i < (int) ulEventNumber; i++) {
             tpe_tss = &pcrEvents[i];
@@ -744,7 +753,7 @@ int getIml(OPENPTS_CONTEXT * ctx, int option) {
                 return -1;  // TODO(munetoh)
             }
             memcpy(tpe, tpe_tss, sizeof(TSS_PCR_EVENT));
-            index = tpe->ulPcrIndex;
+            // index = tpe->ulPcrIndex;
 
             /* copy digest */
             tpe->rgbPcrValue = (BYTE *) malloc(tpe->ulPcrValueLength);
@@ -769,7 +778,7 @@ int getIml(OPENPTS_CONTEXT * ctx, int option) {
             }
 
             /* create wrapper */
-            ew_last = ew_new;
+            // ew_last = ew_new;
             ew_new = (OPENPTS_PCR_EVENT_WRAPPER *)
                 malloc(sizeof(OPENPTS_PCR_EVENT_WRAPPER));
             if (ew_new == NULL) {
@@ -912,7 +921,7 @@ int getBiosImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int mode) {
 
     TSS_PCR_EVENT *event;
     OPENPTS_PCR_EVENT_WRAPPER *ew_new = NULL;
-    OPENPTS_PCR_EVENT_WRAPPER *ew_last = NULL;
+    // OPENPTS_PCR_EVENT_WRAPPER *ew_last = NULL;
     int error = 0;
 
     DEBUG_CAL("getBiosImlFile - start\n");
@@ -1005,7 +1014,7 @@ int getBiosImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int mode) {
 #endif
 
         /* create wrapper */
-        ew_last = ew_new;
+        // ew_last = ew_new;
         ew_new = (OPENPTS_PCR_EVENT_WRAPPER *)
             malloc(sizeof(OPENPTS_PCR_EVENT_WRAPPER));
         if (ew_new == NULL) {
@@ -1281,7 +1290,7 @@ int getImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mod
 
     TSS_PCR_EVENT *event = NULL;
     OPENPTS_PCR_EVENT_WRAPPER *ew = NULL;
-    OPENPTS_PCR_EVENT_WRAPPER *ew_start = NULL;
+    // OPENPTS_PCR_EVENT_WRAPPER *ew_start = NULL;
     OPENPTS_PCR_EVENT_WRAPPER *ew_last = NULL;
 
     DEBUG_CAL("getImaImlFile - start\n");
@@ -1532,7 +1541,7 @@ int getImaImlFile(OPENPTS_CONTEXT * ctx, const char *filename, int type, int mod
         event = NULL;
 
         if (event_num == 0) {
-            ew_start = ew;
+            // ew_start = ew;
             ew_last = ew;
         } else {
             ew->index = event_num;
@@ -1939,7 +1948,7 @@ int printImlByPcr(
     OPENPTS_SNAPSHOT *ss;
     OPENPTS_PCR_EVENT_WRAPPER *eventWrapper;
     char buf[SHA1_BASE64_DIGEST_SIZE + 1];
-    int len;
+    // int len;
 
     /* snapshot */
     ss = getSnapshotFromTable(ctx->ss_table, index, 0);
@@ -1963,10 +1972,10 @@ int printImlByPcr(
         }
 
         /* base64 */
-        len = encodeBase64(
-            (unsigned char *)buf,
-            (unsigned char *)eventWrapper->event->rgbPcrValue,
-            20);
+        // len = encodeBase64(
+        //     (unsigned char *)buf,
+        //     (unsigned char *)eventWrapper->event->rgbPcrValue,
+         //   20);
 
         printf(" (%s) \n", buf);
         eventWrapper = eventWrapper->next_pcr;
