@@ -24,7 +24,7 @@
  * \brief Convert binary IML file to plaintext
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-08-25
- * cleanup 2011-01-22 SM
+ * cleanup 2011-04-26 SM
  *
  * show eventlog (get though TSS)
  *
@@ -82,24 +82,19 @@
 #include <unistd.h>
 #include <stdint.h>
 
-
 #ifndef __STDC_FORMAT_MACROS
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
 #endif
 
-
-//#include <tss/platform.h>
-//#include <tss/tss_defines.h>
-//#include <tss/tss_typedef.h>
-//#include <tss/tss_structs.h>
-//#include <tss/tss_error.h>
-//#include <tss/tspi.h>
-
 #include <openssl/sha.h>
 
 #include <openpts.h>
 
+#ifdef CONFIG_TBOOT
+#include <openpts_tboot.h>
+#endif
+
 // Local TCSD
 #define SERVER    NULL
 
@@ -107,8 +102,18 @@
 // PCR
 BYTE pcr[24][20];
 
+// Verbose Counter
+//       0 no
+// -v    1 show event data contents
+// -v -v 2 DEBUG
+int verbose_cnt = 0;
 int verbose = 0;
 
+char *indent = "                                                              ";
+
+// Check the consistence of IML and PCRs
+int verify = 0;
+
 /*
 
  TPM PCR Read
@@ -121,18 +126,8 @@ int verbose = 0;
 int hex2bin(void *dest, const void *src, size_t n);
 void printhex(char *str, unsigned char *buf, int len);
 
-#if 0
-struct biosEvent {
-    UINT32   pcrIndex;
-    UINT32   eventType;
-    BYTE     digest[20];
-    UINT32   eventDataSize;
-    // BYTE[]   event
-};
-#endif
 
-
-#define BUFSIZE 256
+#define BUFSIZE 4096
 // State
 int pcr4_grub = 0;
 int pcr5_grub = 0;
@@ -218,6 +213,18 @@ void fprintHex(FILE* fp, BYTE* data, UINT32 len) {
     }
 }
 
+
+void fprintChar(FILE* fp, BYTE* data, UINT32 len) {
+    int i;
+    for (i = 0; i < (int)len; i++) {
+        if ((0x20 <= data[i]) && (data[i] < 0x7e)) {
+            fprintf(fp, "%c", data[i]);
+        } else {
+            fprintf(fp, ".");
+        }
+    }
+}
+
 // 61dfe48bca93d211aa0d00e098032b8c 0900000000000000 1a00000000000000
 // 4200 6f00 6f00 7400 4f00 7200 6400 6500 7200 00000100020003000400050006000700080009000a000b000c00]
 void fprintUnicode(FILE *fp, wchar_t * name, int len) {
@@ -230,7 +237,18 @@ void fprintUnicode(FILE *fp, wchar_t * name, int len) {
 }
 
 /**
- *
+ *  
+ */
+void SHA1_UpdateUint32(SHA_CTX *c, UINT32 data) {
+    BYTE buf[4];
+    buf[0] = (data >> 24) & 0xff;
+    buf[1] = (data >> 16) & 0xff;
+    buf[2] = (data >> 8) & 0xff;
+    buf[3] = data & 0xff;
+    SHA1_Update(c, buf, 4);
+}
+
+/**
  *
  * TODO(munetoh) move to iml.c and be common 
  */
@@ -239,7 +257,8 @@ void fprintEventData(
         BYTE* data,  // event
         UINT32 len,  // event length
         UINT32 pcrindex,
-        UINT32 type) {
+        UINT32 type,
+        BYTE*  digest) {
     char buf[BUFSIZE];
     char b64buf[BUFSIZE];
     int i;
@@ -350,6 +369,23 @@ EventData
             break;
         case 8:
             fprintf(fp, "[BIOS:EV_S_CRTM_VERSION]");
+            if (verify == 1) {
+                BYTE digest2[20];
+                SHA_CTX ctx;
+                SHA1_Init(&ctx);
+                SHA1_Update(&ctx, data, len);
+                SHA1_Final(digest2, &ctx);
+                fprintf(fp, "\n                    ");
+                fprintHex(fp, digest2, 20);
+                fprintf(fp, " <= SHA1(Version[%d])", len);
+            }
+            if (verbose_cnt > 0) {
+                fprintf(fp, "\n");
+                fprintf(fp, "%sVersion(hex) : ", indent);
+                fprintHex(fp, data, len);
+                fprintf(fp, "\n%sVersion(char): ", indent);
+                fprintChar(fp, data, len);
+            }
             break;
         case 9:
             fprintf(fp, "[BIOS:EV_CPU_MICROCODE]");
@@ -520,9 +556,10 @@ EventData
         switch (type) {
             /* OpenPTS*/
         case EV_COLLECTOR_START:
-            fprintf(fp, "[OpenPTS:EV_COLLECTOR_START[%d]", len);
-            if (verbose) {
-                fprintf(fp, "=0x");
+            fprintf(fp, "[OpenPTS:EV_COLLECTOR_START[%d]]", len);
+#if 0
+            if (verbose_cnt > 0) {
+                fprintf(fp, "%sUUID");
                 for (i = 0; i < (int)len; i++) {
                     fprintf(fp, "%02x", (BYTE)buf[i]);
                 }
@@ -530,14 +567,150 @@ EventData
                     (unsigned char *)b64buf,
                     (unsigned char *)buf, len);
                 fprintf(fp, ", base64(%s)", b64buf);
-
             }
-            fprintf(fp, "]");
-
+#endif
             break;
         case EV_FILE_SCAN:
             fprintf(fp, "[OpenPTS:EV_FILE_SCAN]");
+            if (verbose_cnt > 0) {
+                OPENPTS_EVENT_FILE_SCAN *fscan;
+                fscan = (OPENPTS_EVENT_FILE_SCAN *) buf;
+                /* show event data */
+                fprintf(fp, "\n");
+                fprintf(fp, "%sFile Mode : %o\n", indent, fscan->file_mode);
+                fprintf(fp, "%sFile UID  : %d\n", indent, fscan->file_uid);
+                fprintf(fp, "%sFile GID  : %d\n", indent, fscan->file_gid);
+                fprintf(fp, "%sFile size : %d\n", indent, fscan->file_size);
+                fprintf(fp, "%sFilename  : %s\n",   indent, fscan->filename);
+                fprintf(fp, "%sDigest    : ",     indent);
+                fprintHex(fp, fscan->digest, 20);
+            }
             break;
+        case EV_FILE_SCAN_TSS:
+            fprintf(fp, "[OpenPTS:EV_FILE_SCAN_TSS]");
+            if (verify == 1) {
+                BYTE digest2[20];
+                SHA_CTX ctx;
+
+                SHA1_Init(&ctx);
+                SHA1_UpdateUint32(&ctx, pcrindex);
+                SHA1_UpdateUint32(&ctx, type);
+                SHA1_Update(&ctx, (BYTE *) data, len);
+                SHA1_Final(digest2, &ctx);
+                fprintf(fp, "\n                    ");
+                fprintHex(fp, digest2, 20);
+                fprintf(fp, " <= SHA1(%d(pcrindex) || %d(eventtype) || eventdata[%d])",
+                    pcrindex, type, len);
+            }
+            if (verbose_cnt > 0) {
+                OPENPTS_EVENT_FILE_SCAN *fscan;
+                fscan = (OPENPTS_EVENT_FILE_SCAN *) buf;
+                /* show event data */
+                fprintf(fp, "\n");
+                fprintf(fp, "%sFile Mode : %o\n", indent, fscan->file_mode);
+                fprintf(fp, "%sFile UID  : %d\n", indent, fscan->file_uid);
+                fprintf(fp, "%sFile GID  : %d\n", indent, fscan->file_gid);
+                fprintf(fp, "%sFile size : %d\n", indent, fscan->file_size);
+                fprintf(fp, "%sFilename  : %s\n",   indent, fscan->filename);
+                fprintf(fp, "%sDigest    : ",     indent);
+                fprintHex(fp, fscan->digest, 20);
+            }
+            break;
+#ifdef CONFIG_TBOOT
+        case EV_TBOOT_SINIT_V6:
+            fprintf(fp, "[tboot:sinit(v6)]");
+            if (verbose_cnt > 0) {
+                OPENPTS_EVENT_TBOOT_SINIT_V6 *ed;
+                ed = (OPENPTS_EVENT_TBOOT_SINIT_V6 *) buf;
+                /* show event data */
+                fprintf(fp, "\n");
+                fprintf(fp, "%ssinit_hash       : ",     indent);
+                fprintHex(fp, ed->sinit_hash, 20);
+                fprintf(fp, "\n");
+                fprintf(fp, "%sedx_senter_flags : ", indent);
+                fprintHex(fp, ed->edx_senter_flags, 4);
+            }
+            break;
+        case EV_TBOOT_SINIT_V7:
+            fprintf(fp, "[tboot:sinit(v7)]");
+            if (verbose_cnt > 0) {
+                OPENPTS_EVENT_TBOOT_SINIT_V7 *ed;
+                ed = (OPENPTS_EVENT_TBOOT_SINIT_V7 *) buf;
+                /* show event data */
+                fprintf(fp, "\n");
+                fprintf(fp, "%ssinit_hash       : ",     indent);
+                fprintHex(fp, ed->sinit_hash, 32);
+                fprintf(fp, "\n");
+                fprintf(fp, "%sedx_senter_flags : ", indent);
+                fprintHex(fp, ed->edx_senter_flags, 4);
+            }
+            break;
+        case EV_TBOOT_STM_V6:
+            fprintf(fp, "[tboot:stm(v6)]");
+            if (verbose_cnt > 0) {
+                OPENPTS_EVENT_TBOOT_STM_V6 *ed;
+                ed = (OPENPTS_EVENT_TBOOT_STM_V6 *) buf;
+                /* show event data */
+                fprintf(fp, "\n");
+                fprintf(fp, "%sbios_acm_id : ",     indent);
+                fprintHex(fp, ed->bios_acm_id, 20);
+                fprintf(fp, "\n");
+                fprintf(fp, "%smseg_valid  : ", indent);
+                fprintHex(fp, ed->mseg_valid, 8);
+            }
+            break;
+        case EV_TBOOT_MLE_HASH:
+            fprintf(fp, "[tboot:mle_hash]");
+            break;
+        case EV_TBOOT_POLCTL:
+            fprintf(fp, "[tboot:tb_policy_control]");
+            if (verbose_cnt > 0) {
+                OPENPTS_EVENT_TBOOT_POLCTL *pc;
+                pc = (OPENPTS_EVENT_TBOOT_POLCTL *) buf;
+                /* show event data */
+                fprintf(fp, "\n");
+                fprintf(fp, "%spolict_control : ", indent);
+                fprintHex(fp, pc->pol_control, 4);
+                fprintf(fp, "\n");
+                fprintf(fp, "%spolicy_hash    : ", indent);
+                fprintHex(fp, pc->pol_hash, 20);
+            }
+            break;
+        case EV_TBOOT_MODULE:
+            fprintf(fp, "[tboot:module]");
+            if (verbose_cnt > 0) {
+                OPENPTS_EVENT_TBOOT_MODULE *ed;
+                ed = (OPENPTS_EVENT_TBOOT_MODULE *) buf;
+                UINT32 size;
+                BYTE *ptr;
+                /* show event data */
+                fprintf(fp, "\n");
+
+                fprintf(fp, "%scommand_hash : ",     indent);
+                fprintHex(fp, ed->command_hash, 20);
+                fprintf(fp, "\n");
+
+                fprintf(fp, "%sfile_hash    : ",     indent);
+                fprintHex(fp, ed->file_hash, 20);
+                fprintf(fp, "\n");
+
+
+                ptr = (BYTE *)&buf[40];
+                size = *(UINT32*) ptr;
+                ptr += 4;
+                fprintf(fp, "%scommand      : ", indent);
+                fprintChar(fp, (BYTE *)ptr, size);
+                fprintf(fp, "\n");
+
+                ptr += size;
+                size = *(UINT32*) ptr;
+                ptr += 4;
+                fprintf(fp, "%sfilename     : ", indent);
+                fprintChar(fp, (BYTE *)ptr, size);
+                fprintf(fp, "\n");
+            }
+            break;
+#endif
         default:
             if ((verbose) && (len < 64)) {
                 fprintf(fp, "[Unknown Event[%d]=0x", len);
@@ -593,26 +766,6 @@ void ima_boot_aggregate(BYTE* digest) {
     SHA1_Final(digest, &ctx);
 }
 
-#if 0
-// move to misc.c
-UINT32 b2l(UINT32 in) {
-    UINT32 out;
-
-    out = in & 0xff;
-    in = in >> 8;
-    out = out << 8;
-    out += in & 0xff;
-    in = in >> 8;
-    out = out << 8;
-    out += in & 0xff;
-    in = in >> 8;
-    out = out << 8;
-    out += in & 0xff;
-
-    return out;
-}
-#endif
-
 /**
  * get IML from file (TCG binary format)
  *
@@ -742,7 +895,10 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
         // DEBUG("\n");
         size = fread(event->rgbEvent, 1, aligned_length, fp);
         if (size != aligned_length) {
-            ERROR("fread NG, size = %d != %d \n", (unsigned int) size, (unsigned int) event->ulEventLength);
+            ERROR("fread NG, size = %d != %d (@PCR[%d])\n",
+                (unsigned int) size,
+                (unsigned int) event->ulEventLength,
+                pcrIndex);
             rc =  TSS_E_INTERNAL_ERROR;
             goto close;
         }
@@ -800,11 +956,6 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
         event = ew->event;
         memcpy(&((*pcr_events)[i]), event, sizeof(TSS_PCR_EVENT));
 
-        /* free */
-        // if (event->rgbEvent != NULL) free(event->rgbEvent);
-        // if (event->rgbPcrValue != NULL) free(event->rgbPcrValue);
-        // free(event);
-
         ew_last = ew;
         ew = ew->next_all;
         free(ew_last);
@@ -822,23 +973,6 @@ TSS_RESULT getEventLog(char *filename, int endian, int aligned, UINT32 *event_nu
         rc = TSS_E_INTERNAL_ERROR;
     }
 
-// free:
-    // TODO(munetoh) we needs free all. but ok at this time:-P
-#if 0
-    if (event != NULL) {
-        if (event->rgbPcrValue != NULL) {
-            free(event->rgbPcrValue);
-        }
-        if (event->rgbEvent != NULL) {
-            free(event->rgbEvent);
-        }
-
-        free(event);  // last one
-    }
-    if (ew  != NULL) {
-        free(ew);  // TODO memory leak
-    }
-#endif
     return rc;  // TSS_E_INTERNAL_ERROR;
 }
 
@@ -852,6 +986,8 @@ void usage(void) {
     fprintf(stderr, "  -i filename           Set binary eventlog file (at securityfs)\n");
     fprintf(stderr, "  -p pcr_index          Select pcr (TSS)\n");
     fprintf(stderr, "  -I mode               Select IMA's log format (Kernel 2.6.32:32)\n");
+    fprintf(stderr, "  -V                    Verify\n");
+    fprintf(stderr, "  -D                    DRTM\n");
     fprintf(stderr, "  -E                    Enable endian conversion (BE->LE or LE->BE)\n");
     fprintf(stderr, "  -h                    Show this help message\n");
     fprintf(stderr, "\n");
@@ -877,24 +1013,26 @@ int main(int argc, char *argv[]) {
     int firmware = 1;
     int kernel = 1;
     int pcrindex = -1;
-    int verify = 0;
     int gmode = 0;  // 0:TSS,1,file
     char *filename = NULL;
     FILE *fp = stdout;
     int c;
     int endian = 0;  // 0:normal 1:convert
     int aligned = 0;
+    int ima_exist = 0;
+    int drtm = 0;
 
-    //
+    /* init */
+    BYTE *blob;
+    UINT32 blobLength;
     memset(zero, 0, 20);
     memset(fox, 0xff, 20);
     memset(boot_aggregate, 0xff, 20);
-    BYTE *blob;
-    UINT32 blobLength;
+
 
 
     /* Args */
-    while ((c = getopt(argc, argv, "i:p:I:EAvh")) != EOF) {
+    while ((c = getopt(argc, argv, "i:p:I:EAvVDh")) != EOF) {
         switch (c) {
         case 'i':
             filename = optarg;
@@ -915,7 +1053,13 @@ int main(int argc, char *argv[]) {
             aligned = 4;
             break;
         case 'v':  /* DEBUG */
-            verbose = DEBUG_FLAG;
+            verbose_cnt++;
+            break;
+        case 'V':  /* verify  */
+            verify = 1;
+            break;
+        case 'D':  /* drtm  */
+            drtm = 1;
             break;
         case 'h':
             usage();
@@ -929,11 +1073,26 @@ int main(int argc, char *argv[]) {
     argc -= optind;
     argv += optind;
 
+    for (i = 0; i < 16; i++) {
+        for (j = 0; j < 20; j++) {
+            pcr[i][j] = 0;
+        }
+    }
+    for (i = 16; i < 24; i++) {
+        /* no DRTM */
+        for (j = 0; j < 20; j++) {
+            if (drtm == 1) pcr[i][j] = 0x00;
+            else           pcr[i][j] = 0xff;
+        }
+    }
 
 
-    if (gmode == 0) {
-        /* TSS */
-        /* Connect to TCSD */
+    if (verbose_cnt > 1) {
+       verbose = DEBUG_FLAG;
+    }
+
+    if (gmode == 0 || verify) {
+        /* in both cases, we have to connect to TCSD */
 
         result = Tspi_Context_Create(&hContext);
         if (result != TSS_SUCCESS) {
@@ -949,7 +1108,6 @@ int main(int argc, char *argv[]) {
             goto close;
         }
 
-
         /* Get TPM handles */
         result = Tspi_Context_GetTpmObject(hContext, &hTPM);
         if (result != TSS_SUCCESS) {
@@ -957,6 +1115,10 @@ int main(int argc, char *argv[]) {
                    result);
             goto close;
         }
+    }
+
+    if (gmode == 0) {
+        /* TSS */
 
         /* Get Log */
 
@@ -1012,7 +1174,8 @@ int main(int argc, char *argv[]) {
                     PcrEvents[i].rgbEvent,
                     PcrEvents[i].ulEventLength,
                     PcrEvents[i].ulPcrIndex,
-                    PcrEvents[i].eventType);
+                    PcrEvents[i].eventType,
+                    PcrEvents[i].rgbPcrValue);
 
                 fprintf(fp, "\n");
 
@@ -1032,6 +1195,9 @@ int main(int argc, char *argv[]) {
                         extend(PcrEvents[i].ulPcrIndex, PcrEvents[i].rgbPcrValue);
                     }
                 }
+                if (PcrEvents[i].ulPcrIndex == 10) {
+                    ima_exist = 1;
+                }
             }
         }
     }
@@ -1047,9 +1213,19 @@ int main(int argc, char *argv[]) {
             for (j = 0; j < 20; j++) {
                 fprintf(fp, "%02x", pcr[i][j]);
             }
-            fprintf(fp, "\t");
             // actual
-            Tspi_TPM_PcrRead(hTPM, i, &blobLength, &blob);
+            result = Tspi_TPM_PcrRead(hTPM, i, &blobLength, &blob);
+            if (result != TSS_SUCCESS) {  // ERROR
+                ERROR("PrcRead failed rc=0x%x\n",
+                       result);
+                goto free;
+            }
+
+            if (memcmp(&pcr[i][0], blob, 20) == 0) {
+                fprintf(fp, "  ==  ");
+            } else {
+                fprintf(fp, "  !=  ");
+            }
 
             for (j = 0; j < 20; j++) {
                 fprintf(fp, "%02x", blob[j]);
@@ -1058,18 +1234,21 @@ int main(int argc, char *argv[]) {
         }
         fprintf(fp, "\n");
         fprintf(fp, "\n");
-        ima_boot_aggregate(digest);
-        fprintf(fp, "IMA boot aggregate:\n");
-        fprintf(fp, "\tcalculated boot_aggregate\t\t\t\tactual boot_aggregate\n");
-        fprintf(fp, "------------------------------------------------------------------------------------------------------\n");
-        for (j = 0; j < 20; j++) {
-            fprintf(fp, "%02x", digest[j]);
+
+        if (ima_exist == 1) {
+            ima_boot_aggregate(digest);
+            fprintf(fp, "IMA boot aggregate:\n");
+            fprintf(fp, "\tcalculated boot_aggregate\t\t\t\tactual boot_aggregate\n");
+            fprintf(fp, "------------------------------------------------------------------------------------------------------\n");
+            for (j = 0; j < 20; j++) {
+                fprintf(fp, "%02x", digest[j]);
+            }
+            fprintf(fp, "\t");
+            for (j = 0; j < 20; j++) {
+                fprintf(fp, "%02x", boot_aggregate[j]);
+            }
+            fprintf(fp, "\n");
         }
-        fprintf(fp, "\t");
-        for (j = 0; j < 20; j++) {
-            fprintf(fp, "%02x", boot_aggregate[j]);
-        }
-        fprintf(fp, "\n");
     }
 
   free:
