@@ -26,7 +26,7 @@
  * \brief Generate Integrity Report from IML
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-04-01
- * cleanup 2011-01-21 SM
+ * cleanup 2011-04-26 SM
  *
  *  TSS Event Struct -> IR
  *  IML and PCR may not match, since the read them is not an atmic operation
@@ -56,12 +56,11 @@
 
 #include <openpts.h>
 
-// #define TSS_V11
-#define TSS_V12
-
 /**
  * free  All FSM in ctx
  *
+ * @return
+ *   PTS_SUCCESS
  */
 int freeAllFsm(OPENPTS_CONTEXT *ctx) {
     OPENPTS_SNAPSHOT *ss;
@@ -115,9 +114,8 @@ int freeAllFsm(OPENPTS_CONTEXT *ctx) {
 /**
  * New IR context
  *
- * Return
- *   OPENPTS_IR_CONTEXT
- *   NULL
+ * @return
+ *   pointer to OPENPTS_IR_CONTEXT OR NULL
  *
  */
 OPENPTS_IR_CONTEXT *newIrContext() {
@@ -136,12 +134,16 @@ OPENPTS_IR_CONTEXT *newIrContext() {
         free(ctx);
         return NULL;
     }
+    memset(ctx->buf, 0, EVENTDATA_BUF_SIZE);
 
     return ctx;
 }
 
 /**
  * Free IT Context
+ *
+ * @return void
+ *
  */
 void freeIrContext(OPENPTS_IR_CONTEXT *ctx) {
     if (ctx == NULL) {
@@ -163,9 +165,9 @@ void freeIrContext(OPENPTS_IR_CONTEXT *ctx) {
  * To supports stacked component, we need FSM model to parse the IML.
  * But, if we use the FSM model, verifier can validate the IML without stacked component
  *
- * Return
- *   PTS_SUCCESS
- *   PTS_INTERNAL_ERROR
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
+ *
  *
  */
 int writeComponentID(
@@ -322,9 +324,8 @@ int writeComponentID(
 /**
  * write core:DigestMethod
  *
- * Return
- *   PTS_SUCCESS
- *   PTS_INTERNAL_ERROR
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  *
  */
 int writeDigestMethod(xmlTextWriterPtr writer) {
@@ -380,9 +381,8 @@ int writeDigestMethod(xmlTextWriterPtr writer) {
  * @param ss_level
  * @param eventindex
  *
- * Return
- *   PTS_SUCCESS
- *   PTS_INTERNAL_ERROR
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  *
  *
  */
@@ -503,9 +503,8 @@ int writeStuffObjects(
   j7/z7OqcVMjRxCz+qT1r8BvzQFs=
 </PcrHash>
  *
- * Return
- *   PTS_SUCCESS
- *   PTS_INTERNAL_ERROR
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  *
  */
 
@@ -579,9 +578,8 @@ int writePcrHash(
 /**
  * write Snapshot
  *
- * Return
- *  PTS_SUCCESS
- *  PTS_INTERNAL_ERROR
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  */
 int writeSnapshot(
         xmlTextWriterPtr writer,
@@ -600,6 +598,14 @@ int writeSnapshot(
 
     OPENPTS_PCR_EVENT_WRAPPER *eventWrapper;
 
+    /* reset PCR */
+    // FSM resetPCR(n) exist
+    if (ss->reset_pcr == 1) {
+        TODO("reset PCR[%d]\n", index);
+        resetTpmPcr(tpm, index);
+    }
+
+
     /* set initial PCR value */
     rc = getTpmPcrValue(tpm, index, ss->start_pcr);
 
@@ -747,9 +753,8 @@ int writeSnapshot(
  * write Quote
  *
  *
- * Return
- *   PTS_SUCCESS
- *   PTS_INTERNAL_ERROR
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  *
  */
 /*
@@ -1108,6 +1113,9 @@ int writeQuote(
  *       ExtendedData - base64
  *       PcrInfoShort
  *     CapVersioninfoType
+ *
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  */
 int writeQuote2(
         xmlTextWriterPtr writer,
@@ -1120,7 +1128,7 @@ int writeQuote2(
     int tag;
     char fixed[5];
     int locality;
-    BYTE *external_data;
+    // BYTE *external_data;
     BYTE *composite_hash;
 
     if (ctx->pcrs == NULL) {
@@ -1143,7 +1151,7 @@ int writeQuote2(
 
 
     /* Quote2 - Nonce [6:25] */
-    external_data = &ctx->validation_data->rgbData[6];
+    // external_data = &ctx->validation_data->rgbData[6];
 
     /* Quote2 - PcrSelection [26:27] */
     size_of_select = ctx->validation_data->rgbData[27];
@@ -1464,25 +1472,23 @@ int writeQuote2(
  * return 0:success, -1:error
  *  TODO PTS_SUCCESS PTS_XXX
  *
- * Return
- *   PTS_SUCCESS
- *   PTS_INTERNAL_ERROR
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  *
  */
+// TODO remove file
 int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
+// int writeIr(OPENPTS_CONTEXT *ctx) {
     int rc = PTS_SUCCESS;
-    int i, j;
+    int i;
     xmlTextWriterPtr writer;
     xmlBufferPtr buf;
     FILE *fp;
-    // DUMMY
-    BYTE dymmyHash[20];
     PTS_ComponentId cid;
-    TSS_PCR_EVENT event;
     OPENPTS_TPM_CONTEXT tpm;  // to calc snapshot PCR
-
     PTS_UUID *ir_uuid;
     char *str_ir_uuid;
+    char filename[256];  // TODO UUID_UUID.xml
 
     PTS_Byte smbios[12] = { 0x4A, 0x4A, 0x4A, 0x4A, 0x4A,
                             0x4A, 0x4A, 0x4A, 0x4A, 0x4A,
@@ -1530,16 +1536,8 @@ int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
     cid.dataBlock.blockSize = 0;
     cid.dataBlock.dataBlock = smbios;
 
-
-    event.ulPcrIndex = 0;
-    event.eventType = 0;
-    event.ulPcrValueLength = 20;
-    event.rgbPcrValue = dymmyHash;
-    event.ulEventLength = 20;
-    event.rgbEvent = dymmyHash;
-
     /* reset TPM */
-    resetTpm(&tpm);
+    resetTpm(&tpm, ctx->drtm);
 
     /* Create a new XML buffer */
     buf = xmlBufferCreate();
@@ -1665,6 +1663,24 @@ int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
         if (ctx->conf->ir_without_quote == 1) {
             TODO("skip TPM_Quote\n");
         } else {
+            if (ctx->conf->tpm_quote_type == 1) {
+                /* Quote */
+                rc = writeQuote(writer, ctx);
+                if (rc < 0) {
+                    ERROR("writeIr - writeQuote() rc = %d\n", rc);
+                    rc = PTS_INTERNAL_ERROR;
+                    goto free;
+                }
+            } else {
+                /* Quote2 */
+                rc = writeQuote2(writer, ctx);
+                if (rc < 0) {
+                    ERROR("writeIr - writeQuote2() rc = %d\n", rc);
+                    rc = PTS_INTERNAL_ERROR;
+                    goto free;
+                }
+            }
+#if 0
 #ifdef TSS_V11
             rc = writeQuote(writer, ctx);
             if (rc < 0) {
@@ -1679,6 +1695,7 @@ int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
                 rc = PTS_INTERNAL_ERROR;
                 goto free;
             }
+#endif
 #endif
         }
     } else {
@@ -1687,7 +1704,7 @@ int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
 
     /* add Snapshots */
     DEBUG_CAL("genIr - start snapshot\n");
-    j = 0;
+    // j = 0;
 
     for (i = 0; i < MAX_PCRNUM; i++) {
         /* level 0, platform */
@@ -1711,6 +1728,12 @@ int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
         }
     }  // PCR LOOP
 
+    rc = xmlTextWriterFlush(writer);
+    if (rc < 0) {
+        ERROR("writeRm: Error at xmlTextWriterFlush\n");
+        rc = PTS_INTERNAL_ERROR;
+        goto free;
+    }
 
     /* Close all elements */
     rc = xmlTextWriterEndDocument(writer);
@@ -1720,12 +1743,29 @@ int writeIr(OPENPTS_CONTEXT *ctx, const char *file) {
         goto free;
     }
 
+
+    /* set IR file */
+    // TODO function
+    if (ctx->conf->ir_filename != NULL) {
+        free(ctx->conf->ir_filename);
+    }
+    if (ctx->conf->ir_dir == NULL) {
+        ERROR("Set ir.dir at ptsc.conf. \n");
+        ctx->conf->ir_dir = smalloc("/tmp/.ptsc");
+    }
+    snprintf(filename, sizeof(filename), "%s_%s.xml",
+        ctx->str_uuid,
+        str_ir_uuid);
+
+    ctx->conf->ir_filename = getFullpathName(ctx->conf->ir_dir, filename);
+
+
     /* write to file */
     xmlFreeTextWriter(writer);
 
-    fp = fopen(file, "w");
+    fp = fopen(ctx->conf->ir_filename, "w");
     if (fp == NULL) {
-        ERROR("testXmlwriterMemory: Error at fopen\n");
+        ERROR("testXmlwriterMemory: Error at fopen, %s\n", ctx->conf->ir_filename);
         rc = PTS_INTERNAL_ERROR;
         goto free;
     }
@@ -1934,6 +1974,10 @@ void  irStartElement(void* ctx, const xmlChar* name, const xmlChar** atts) {
         }
         /* set pcr_select */
         if (pcrs->pcr_select_size > 0) {
+            // ir_ctx->pcrs->pcr_select_byte
+            if (pcrs->pcr_select_byte != NULL) {
+                free(pcrs->pcr_select_byte);
+            }
             pcrs->pcr_select_byte = malloc(pcrs->pcr_select_size);
             if (pcrs->pcr_select_byte == NULL) {
                 ERROR("no memory\n");
@@ -2028,6 +2072,9 @@ void  irStartElement(void* ctx, const xmlChar* name, const xmlChar** atts) {
                             strlen(value));
                         // TODO check rc
                         validation_data->ulExternalDataLength = rc;
+                        if (validation_data->rgbExternalData != NULL) {
+                            free(validation_data->rgbExternalData);
+                        }
                         validation_data->rgbExternalData = malloc(rc);
                         memcpy(validation_data->rgbExternalData, buf, rc);
                         memcpy(&validation_data->rgbData[28], buf, 20);
@@ -2109,11 +2156,12 @@ void irEndElement(void * ctx, const xmlChar * name) {
     TSS_VALIDATION *validation_data = pctx->validation_data;
     OPENPTS_PCRS *pcrs = pctx->pcrs;
     int rc;
-    BYTE buf[IR_SAX_BUFFER_SIZE];  // TODO
-
-    // printf("Element=%s char size=%d\n", name, ir_ctx->char_size);
+    BYTE buf[IR_SAX_BUFFER_SIZE];
 
     if (!strcmp((char *)name, "stuff:Objects")) {
+        int extend = 0;
+        int pcr_index = -1;
+        BYTE* digest = NULL;
         /* Event finish, let's get into our structure */
 
         ir_ctx->event_index++;
@@ -2136,24 +2184,32 @@ void irEndElement(void * ctx, const xmlChar * name) {
             ir_ctx->ew_new->event = NULL;
             ir_ctx->sax_error++;
         } else {
-            /* TPM extend  */
-            extendTpm2(
-                &pctx->tpm,
-                ir_ctx->event->ulPcrIndex,
-                ir_ctx->event->rgbPcrValue);
+            /* TPM extend - after FSM push  */
+            extend = 1;
+            pcr_index = ir_ctx->event->ulPcrIndex;
+            digest = ir_ctx->event->rgbPcrValue;
 
             /* move to the EW chain */
             ir_ctx->ew_new->event = ir_ctx->event;
             ir_ctx->event = NULL;
         }
 
-        /* map to the snapshot */
+        /* map to the snapshot, push FSM  */
         rc = addEventToSnapshotBin(pctx, ir_ctx->ew_new);  // iml.c
         if (rc != PTS_SUCCESS) {
             // ERROR("validateIr:irStartElement - addEventToSnapshotBin rc = %d\n", rc);
             ir_ctx->integrity = OPENPTS_RESULT_INVALID;
             return;
         }
+        /* extend after FSM push. after execution of Action() in FSM */
+        if (extend == 1) {
+            extendTpm2(
+                &pctx->tpm,
+                pcr_index,
+                digest);
+        }
+
+
     } else if (!strcmp((char *)name, "SnapshotCollection")) {
         /*  snapshot finish  */
         /* Push FSM until Final state to run actions */
@@ -2200,6 +2256,14 @@ void irEndElement(void * ctx, const xmlChar * name) {
         if (rc != 0) {
             ERROR("ERROR PCR[%d] != IML\n", ir_ctx->pcr_index);
             ir_ctx->sax_error = 1;
+            // verbose = DEBUG_FLAG | DEBUG_TPM_FLAG;  // switch DEBUG MODE
+            if (verbose & DEBUG_FLAG) {
+                BYTE pcr[20];
+                DEBUG("PCR[%d]\n", ir_ctx->pcr_index);
+                getTpmPcrValue(&pctx->tpm, ir_ctx->pcr_index, pcr);
+                printHex("", pcr, 20, " (emulated)\n");
+                printHex("", ir_ctx->pcr, 20, " (IR)\n");
+            }
         } else {
             /* IML and PCR are consistent :-) */
             DEBUG_FSM("PCR[%d] == IML\n", ir_ctx->pcr_index);
@@ -2232,8 +2296,9 @@ void irEndElement(void * ctx, const xmlChar * name) {
         rc = decodeBase64(
             buf,
             (unsigned char *)ir_ctx->buf,
-            strlen(ir_ctx->buf));
+            ir_ctx->char_size);  //strlen(ir_ctx->buf));
         // TODO check rc
+        // TODO("CompositeHash %s", ir_ctx->buf);
         memcpy(&validation_data->rgbData[32], buf, 20);
     } else if (!strcmp((char *)name, "ValueSize")) {
         // <ValueSize>200</ValueSize>
@@ -2277,6 +2342,9 @@ void irEndElement(void * ctx, const xmlChar * name) {
                     ir_ctx->char_size);
             // TODO check rc
             validation_data->ulValidationDataLength = rc;
+            if (validation_data->rgbValidationData != NULL) {
+                free(validation_data->rgbValidationData);
+            }
             validation_data->rgbValidationData = malloc(rc);
             memcpy(validation_data->rgbValidationData, buf, rc);
             // DEBUG("rgbValidationData =%s\n",ir_ctx->buf);
@@ -2352,12 +2420,11 @@ void irCharacters(void* ctx, const xmlChar * ch, int len) {
  * @param ctx PTS_CONTEXT
  * @param filename IR file
  *
- * Return 
- *   OPENPTS_RESULT_VALID
- *   OPENPTS_RESULT_INVALID
- *   OPENPTS_RESULT_UNKNOWN
+ * @retval OPENPTS_RESULT_VALID
+ * @retval OPENPTS_RESULT_INVALID
+ * @retval OPENPTS_RESULT_UNKNOWN
  *
- *   -1 ERROR
+ * @retval PTS_FATAL
  *
  * Capability is Limited at this moment.
  * - check the consistency between IML and PCR
@@ -2380,7 +2447,7 @@ void irCharacters(void* ctx, const xmlChar * ch, int len) {
  */
 int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
     xmlSAXHandler  sax_handler;
-    int rc = 0;
+    int rc = PTS_SUCCESS;
     OPENPTS_CONFIG *conf;
     OPENPTS_PCRS *pcrs;
 
@@ -2390,11 +2457,11 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
     /* check */
     if (ctx == NULL) {
         ERROR("ctx == NULL\n");
-        return PTS_INTERNAL_ERROR;
+        return PTS_FATAL;
     }
     if (ctx->target_conf == NULL) {
         ERROR("ctx->target_conf == NULL\n");
-        return PTS_INTERNAL_ERROR;
+        return PTS_FATAL;
     }
     conf = ctx->target_conf;
 
@@ -2403,7 +2470,7 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
         ctx->pcrs = malloc(sizeof(OPENPTS_PCRS));
         if (ctx->pcrs == NULL) {
             ERROR("no memory\n");
-            return PTS_INTERNAL_ERROR;
+            return PTS_FATAL;
         }
         memset(ctx->pcrs, 0, sizeof(OPENPTS_PCRS));
     }
@@ -2416,7 +2483,7 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
         ctx->ir_ctx = newIrContext();
         if (ctx->ir_ctx == NULL) {
             ERROR("no memory\n");
-            return -1;
+            return PTS_FATAL;
         }
     }
 
@@ -2441,7 +2508,7 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
             rc = loadSQLiteDatabaseFile(ctx->aide_ctx, conf->aide_sqlite_filename);
             if (rc != PTS_SUCCESS) {
                 ERROR("loadSQLiteDatabaseFile fail\n");
-                return -1;  // TODO
+                return PTS_FATAL;
             }
 #else
             rc = loadAideDatabaseFile(ctx->aide_ctx, conf->aide_database_filename);
@@ -2496,11 +2563,14 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
             ctx->pcrs->pubkey_length = conf->pubkey_length;
             ctx->pcrs->pubkey = conf->pubkey;
 
-#ifdef TSS_V11
-            rc = validatePcrCompositeV11(ctx->pcrs, ctx->validation_data);  // tss.c
-#else
-            rc = validatePcrCompositeV12(ctx->pcrs, ctx->validation_data);
-#endif
+            if (ctx->conf->tpm_quote_type == 1) {
+                /* Quote */
+                rc = validatePcrCompositeV11(ctx->pcrs, ctx->validation_data);  // tss.c
+            } else {
+                /* Quote2 */
+                rc = validatePcrCompositeV12(ctx->pcrs, ctx->validation_data);
+            }
+
             DEBUG("validateIr() - validatePcrComposite, set rc=%d\n", rc);
             // DBEUG("validatePcrCompositeV11 = %d\n",rc);
             if (rc == PTS_SUCCESS) {
@@ -2508,7 +2578,7 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
             } else if (rc == PTS_VERIFY_FAILED) {
                 // ptscd - if FSM config in ptscd.conf is wrong this happen
                 addReason(ctx,
-                    "[QUOTE] verification of PCR Composite was failed, (tscd - bad FSM configulation in ptscd.conf)");
+                    "[QUOTE] verification of PCR Composite was failed, (tscd - bad FSM configuration in ptscd.conf)");
                 addProperty(ctx, "tpm.quote.pcrs", "invalid");
             } else {
                 addReason(ctx,
@@ -2585,6 +2655,9 @@ int validateIr(OPENPTS_CONTEXT *ctx, const char *filename) {
 
 /**
  *  gen IR from Securityfs
+ *
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  */
 int genIrFromSecurityfs(OPENPTS_CONTEXT *ctx) {
     int rc;
@@ -2598,7 +2671,7 @@ int genIrFromSecurityfs(OPENPTS_CONTEXT *ctx) {
 #endif
 
     /* reset TPM emu */
-    resetTpm(&ctx->tpm);
+    resetTpm(&ctx->tpm, ctx->drtm);
 
     /* reset FSM */
     rc = freeAllFsm(ctx);
@@ -2606,15 +2679,15 @@ int genIrFromSecurityfs(OPENPTS_CONTEXT *ctx) {
     /* setup FSM */
     rc = readFsmFromPropFile(ctx, ctx->conf->config_file);
     if (rc != PTS_SUCCESS) {
-        ERROR("\n");
-        return -1;
+        ERROR("readFsmFromPropFile %s failed\n", ctx->conf->config_file);
+        return PTS_INTERNAL_ERROR;
     }
 
     /* read BIOS IML */
     rc = getBiosImlFile(ctx, ctx->conf->bios_iml_filename, ctx->conf->iml_endian);
     if (rc != PTS_SUCCESS) {
         ERROR("fail to load BIOS IML, rc = %d\n", rc);
-        return -1;
+        return PTS_INTERNAL_ERROR;
     }
 
     if (ctx->conf->runtime_iml_filename != NULL) {
@@ -2622,7 +2695,7 @@ int genIrFromSecurityfs(OPENPTS_CONTEXT *ctx) {
         rc = getImaImlFile(ctx, ctx->conf->runtime_iml_filename, ctx->conf->runtime_iml_type, 0);  // TODO endian?
         if (rc < 0) {
             ERROR("fail to load IMA IML, rc = %d\n", rc);
-            return -1;
+            return PTS_INTERNAL_ERROR;
         }
     }
 
@@ -2638,7 +2711,7 @@ int genIrFromSecurityfs(OPENPTS_CONTEXT *ctx) {
     rc = writeIr(ctx, ctx->conf->ir_filename);
     if (rc != 0) {
         ERROR("fail to write IR, rc = %d\n", rc);
-        return -1;
+        return PTS_INTERNAL_ERROR;
     }
 
     return PTS_SUCCESS;
@@ -2646,7 +2719,10 @@ int genIrFromSecurityfs(OPENPTS_CONTEXT *ctx) {
 
 
 /**
- *  gen IR from Securityfs
+ * gen IR from Securityfs
+ *
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR 
  */
 int genIrFromTss(OPENPTS_CONTEXT *ctx) {
     int rc;
@@ -2655,7 +2731,7 @@ int genIrFromTss(OPENPTS_CONTEXT *ctx) {
     /* get IML via securityfs */
 
     /* reset TPM emu */
-    resetTpm(&ctx->tpm);
+    resetTpm(&ctx->tpm, ctx->drtm);
 
     /* reset FSM */
     rc = freeAllFsm(ctx);
@@ -2672,7 +2748,7 @@ int genIrFromTss(OPENPTS_CONTEXT *ctx) {
     rc = readFsmFromPropFile(ctx, ctx->conf->config_file);  // fsm.c
     if (rc != PTS_SUCCESS) {
         ERROR("read FSM failed\n");
-        return -1;
+        return PTS_INTERNAL_ERROR;
     }
 
     /* TSS_VALIDATION */
@@ -2700,27 +2776,27 @@ int genIrFromTss(OPENPTS_CONTEXT *ctx) {
     if (ctx->conf->ir_without_quote == 1) {
         TODO("skip TPM_Quote\n");
     } else {
-        /* TPM Quote */
-#ifdef TSS_V11
-        rc = quoteTss(
-                ctx->conf->uuid->uuid,
-                ps_type,
-                ctx->conf->srk_password_mode,
-                NULL, NULL,
-                ctx->pcrs,
-                ctx->validation_data);  // tss.c
-#else
-        rc = quote2Tss(
-                ctx->conf->uuid->uuid,
-                ps_type,
-                ctx->conf->srk_password_mode,
-                NULL, NULL,
-                ctx->pcrs,
-                ctx->validation_data);  // tss.c
-#endif
+        /* TPM Quote or TPM Quote2 */
+        if (ctx->conf->tpm_quote_type == 1) {
+            rc = quoteTss(
+                    ctx->conf->uuid->uuid,
+                    ps_type,
+                    ctx->conf->srk_password_mode,
+                    NULL, NULL,
+                    ctx->pcrs,
+                    ctx->validation_data);  // tss.c
+        } else {
+            rc = quote2Tss(
+                    ctx->conf->uuid->uuid,
+                    ps_type,
+                    ctx->conf->srk_password_mode,
+                    NULL, NULL,
+                    ctx->pcrs,
+                    ctx->validation_data);  // tss.c
+        }
         if (rc != 0) {
             ERROR("quoteTss fail, rc = 0x%04d\n", rc);
-            return -1;
+            return PTS_INTERNAL_ERROR;
         }
     }
 
@@ -2728,35 +2804,38 @@ int genIrFromTss(OPENPTS_CONTEXT *ctx) {
     rc = setPcrsToSnapshot(ctx, ctx->pcrs);  // TODO
     if (rc < 0) {
         ERROR("fail to load PCR, rc = %d\n", rc);
-        return -1;
+        return PTS_INTERNAL_ERROR;
     }
 
     /* get BIOS/IMA IML */
     rc = getIml(ctx, 0);
     if (rc < 0) {
         ERROR("fail to load BIOS IML, rc = %d\n", rc);
-        return -1;
+        return PTS_INTERNAL_ERROR;
     }
 
     /* save IR */
     rc = writeIr(ctx, ctx->conf->ir_filename);  // ir.c
     if (rc != 0) {
         ERROR("fail to write IR, rc = %d\n", rc);
-        return -1;
+        return PTS_INTERNAL_ERROR;
     }
 
-    return 0;
+    return PTS_SUCCESS;
 }
 
 
 /**
  *  gen IR file
+ *
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  */
 int genIr(OPENPTS_CONTEXT *ctx) {
     int rc = PTS_INTERNAL_ERROR;
     if (ctx->conf->iml_mode == 1) {
         rc = genIrFromSecurityfs(ctx);
-        if (rc != 0) {
+        if (rc != PTS_SUCCESS) {
             ERROR("writePtsTlvToSock - gen IR failed\n");
             return rc;
         }
@@ -2766,7 +2845,7 @@ int genIr(OPENPTS_CONTEXT *ctx) {
 #else
         // DEBUG("get IML/PCR via TSS is not ready\n");
         rc = genIrFromTss(ctx);
-        if (rc != 0) {
+        if (rc != PTS_SUCCESS) {
             ERROR("gen IR failed\n");
             return rc;
         }
