@@ -26,56 +26,11 @@
  * \brief main of openpts command
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-07-25
- * cleanup 2011-01-21 SM
+ * cleanup 2011-04-26 SM
  *
  * This is verifier and utility to maintain the collector/verifier
  *
  *
- * Local host (for TEST)
- *
- * e.g.
- * $ ../src/openpts -i -f -p 5557 localhost
- * target            : localhost
- * configulation     : /home/munetoh/.openpts/07139e72-1ef7-11e0-9833-001f160c9c28/target.conf
- * manifest[0]       : /home/munetoh/.openpts/07139e72-1ef7-11e0-9833-001f160c9c28//074c9722-1ef7-11e0-9833-001f160c9c28/rm0.xml
- * manifest[1]       : /home/munetoh/.openpts/07139e72-1ef7-11e0-9833-001f160c9c28//074c9722-1ef7-11e0-9833-001f160c9c28/rm1.xml
- * validation policy : /home/munetoh/.openpts/07139e72-1ef7-11e0-9833-001f160c9c28/policy.conf
- * 
- * $ ../src/openpts localhost
- * target        : localhost
- * port          : 5557 (localhost)
- * policy file   : /home/munetoh/.openpts/07139e72-1ef7-11e0-9833-001f160c9c28/policy.conf
- * property file : /home/munetoh/.openpts/07139e72-1ef7-11e0-9833-001f160c9c28/vr.property
- * integrity     : valid
- *
- *
- * Remote host w/ SSH port forword
- *
- *  Verifier |            SSH tunneling         |    Collector
- *
- *  openpts-->localhost:5557
- *                       ssh -------> sshd:22
- *                                       sshd-->localhost:5556
- *                                                      =ptscd
- * 
- * e.g.
- * $ ./src/openpts -i -s -S 5567 -l munetoh -p 5557 localhost
- * munetoh@localhost's password: 
- * target            : localhost
- * configulation     : /home/munetoh/.openpts/07139e72-1ef7-11e0-9833-001f160c9c28/target.conf
- * manifest[0]       : /home/munetoh/.openpts/07139e72-1ef7-11e0-9833-001f160c9c28//074c9722-1ef7-11e0-9833-001f160c9c28/rm0.xml
- * manifest[1]       : /home/munetoh/.openpts/07139e72-1ef7-11e0-9833-001f160c9c28//074c9722-1ef7-11e0-9833-001f160c9c28/rm1.xml
- * validation policy : /home/munetoh/.openpts/07139e72-1ef7-11e0-9833-001f160c9c28/policy.conf
- *
- * ./src/openpts -localhost
- * target        : localhost
- * port          : 5557 (remotehost)
- * port(ssh)     : 5567 (localhost)
- * username(ssh) : munetoh
- * policy file   : /home/munetoh/.openpts/07139e72-1ef7-11e0-9833-001f160c9c28/policy.conf
- * property file : /home/munetoh/.openpts/07139e72-1ef7-11e0-9833-001f160c9c28/vr.property
- * integrity     : valid
- *
  *
  */
 
@@ -102,6 +57,12 @@
 
 int verbose = 0; /**< DEBUG */
 
+// DEBUG
+// /usr/sbin/ptsc -m
+// /PATH/ptsc -m -c XXX
+// openpts -P PATH -C CONF
+extern char *ptsc_command;
+
 #define LINE "--------------------------------------------------------------------"
 
 /**
@@ -109,23 +70,25 @@ int verbose = 0; /**< DEBUG */
  */
 void usage(void) {
     fprintf(stderr, "OpenPTS command\n\n");
-    fprintf(stderr, "Usage: openpts [options] [command] target\n\n");
+    fprintf(stderr, "Usage: openpts [options] {-i [-f]|[-v]|-D} <target>\n");
+    fprintf(stderr, "       openpts -D\n\n");
     fprintf(stderr, "Commands:\n");
-    fprintf(stderr, "  -i                    Initialize PTS verifier with target(collector)\n");
-    fprintf(stderr, "  -u                    Update PTS verifier with target(collector)\n");
-    fprintf(stderr, "  -D                    Display the configulation (ALL)\n");
+    fprintf(stderr, "  -i [-f]               Initialize [forcibly] the PTS verifier with the target(collector).\n");
+    fprintf(stderr, "  [-v]                  Verify target(collector) integrity against know measure.\n");
+    fprintf(stderr, "  -D                    Display the configuration (target/ALL)\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Miscellaneous:\n");
     fprintf(stderr, "  -h                    Show this help message\n");
-    fprintf(stderr, "  -v                    Verbose mode. Multiple -v options increase the verbosity.\n");
+    fprintf(stderr, "  -V                    Verbose mode. Multiple -V options increase the verbosity.\n");
     fprintf(stderr, "\n");
     fprintf(stderr, "Options:\n");
-    fprintf(stderr, "  -p port               Set port number. default is %d\n", PTSCD_PORT);
-    fprintf(stderr, "  -c configfile         Set configulation file. defalt is %s\n", PTSCD_CONFIG_FILE);
-    fprintf(stderr, "  -f                    Force init, delete existing target(collector) info\n");
-    fprintf(stderr, "  -s                    use SSH tunnel (port forward)\n");
-    fprintf(stderr, "  -S port               setup SSH tunnel, set local port\n");
-    fprintf(stderr, "  -l username           ssh username\n");
+#ifdef CONFIG_AUTO_RM_UPDATE
+    fprintf(stderr, "  -u                    "
+                    "Selects 'yes' as the the default answer when an update is available [no]\n");
+#endif
+    fprintf(stderr, "  -l username           ssh username [ssh default]\n");
+    fprintf(stderr, "  -p port               ssh port number [ssh default]\n");
+    fprintf(stderr, "  -c configfile         Set configuration file [~/.openpts/openpts.conf]\n");
     fprintf(stderr, "\n");
 }
 
@@ -133,6 +96,7 @@ void usage(void) {
 #define VERIFY  1
 #define UPDATE  2
 #define DISPLAY 3
+#define NONE    4
 
 /**
  * print target info
@@ -159,13 +123,11 @@ int printTargetInfo(OPENPTS_CONFIG *conf, char *conf_dir, char *uuid, int cnt) {
     printf("target[%d] uuid            : %s\n", cnt, uuid);
     printf("target[%d] config          : %s\n", cnt, target_conf_filename);
     printf("target[%d] hostname        : %s\n", cnt, conf->hostname);
-    printf("target[%d] port            : %d\n", cnt, conf->port);
-    if (conf->ssh_mode == OPENPTS_SSH_MODE_ON) {
-        printf("target[%d] SSH             : on\n", cnt);
+    if (conf->ssh_username != NULL) {
         printf("target[%d] SSH remote user : %s\n", cnt, conf->ssh_username);
-        printf("target[%d] SSH local port  : %d\n", cnt, conf->ssh_port);
-    } else {
-        printf("target[%d] SSH             : off\n", cnt);
+    }
+    if (conf->ssh_port != NULL) {
+        printf("target[%d] SSH port        : %s\n", cnt, conf->ssh_port);
     }
 
     return PTS_SUCCESS;
@@ -181,6 +143,9 @@ int printTargetInfo(OPENPTS_CONFIG *conf, char *conf_dir, char *uuid, int cnt) {
 // TODO use selectUuidDir in uuid.c
 static int selectDir(const struct dirent *entry) {
     int len;
+#ifndef __linux__
+    struct stat buffer;
+#endif
 
     /* skip . .. dirs */
     if (0 == strcmp(".", entry->d_name)) return 0;
@@ -193,7 +158,12 @@ static int selectDir(const struct dirent *entry) {
     if (len != 36) return 0;
 
     /* Dir HIT */
+#ifndef __linux__
+    if (stat(entry->d_name, &buffer) != 0) return 0;
+    if (S_ISDIR(buffer.st_mode)) return 1;
+#else
     if (entry->d_type == DT_DIR) return 1;
+#endif
 
     return 0;
 }
@@ -208,50 +178,65 @@ static int selectDir(const struct dirent *entry) {
  *
  */
 int main(int argc, char *argv[]) {
-    int command = VERIFY;
+    int command = NONE;
     int rc = 0;
-    int c;
-    int port = PTSCD_PORT;
+    int opt;
 
     OPENPTS_CONFIG *conf = NULL;  // conf for openpts
     OPENPTS_CONTEXT *ctx = NULL;
-    char * execpath = NULL;
+    // char * execpath = NULL;
     char * config_filename = NULL;
     char * target_hostname = NULL;
     int initialized = 0;  // 0 -> 1 -> 2
     char * target_conf_dir = NULL;
     char * target_conf_filename = NULL;
     int force = 0;
-    int ssh = 0;
-    int ssh_config = 0;
-    int ssh_port = 0;
-    char *ssh_user = NULL;
+    char *ssh_port = NULL;
+    char *ssh_username = NULL;
+#ifdef CONFIG_AUTO_RM_UPDATE
+    int update_by_default = 0;
+#endif
     int i;
     OPENPTS_TARGET *target_collector = NULL;
     OPENPTS_CONFIG *target_conf = NULL;    // conf for target
     int new_target = 0;  // indicate new target
     int debug = 0;
+    // DEBUG
+    char *ptsc_path = NULL;
+    char *ptsc_conf = NULL;
 
     verbose = 0;
-    execpath = argv[0];
+    // execpath = argv[0];
 
     /* args */
-    while ((c = getopt(argc, argv, "ivuDp:c:dfsS:l:h")) != EOF) {
-        switch (c) {
+    while ((opt = getopt(argc, argv, "ivuDVc:dfuyl:p:P:C:h")) != -1) {
+        switch (opt) {
         case 'i':
-            command = INIT;
-            break;
+            if (command == NONE) {
+                command = INIT;
+                break;
+            }
+            ERROR("Only one command may be given at a time.");
+            usage();
+            return -1;
         case 'v':
-            debug++;
-            break;
-        case 'u':
-            command = UPDATE;
-            break;
+            if (command == NONE) {
+                command = VERIFY;
+                break;
+            }
+            ERROR("Only one command may be given at a time.");
+            usage();
+            return -1;
         case 'D':
-            command = DISPLAY;
-            break;
-        case 'p':
-            port = atoi(optarg);
+            if (command == NONE) {
+                command = DISPLAY;
+                break;
+            }
+            ERROR("Only one command may be given at a time.");
+            usage();
+            return -1;
+        case 'V':
+            debug++;
             break;
         case 'c':
             config_filename = optarg;
@@ -262,17 +247,22 @@ int main(int argc, char *argv[]) {
         case 'f':
             force = 1;
             break;
-        case 's':
-            ssh = 1;
-            ssh_config = 1;
-            break;
-        case 'S':
-            ssh = 1;
-            ssh_config = 1;
-            ssh_port = atoi(optarg);
+#ifdef CONFIG_AUTO_RM_UPDATE
+        case 'u':
+            update_by_default = 1;
             break;
+#endif
         case 'l':
-            ssh_user = optarg;
+            ssh_username = optarg;
+            break;
+        case 'p':
+            ssh_port = optarg;
+            break;
+        case 'P':
+            ptsc_path = optarg;
+            break;
+        case 'C':
+            ptsc_conf = optarg;
             break;
         case 'h':
             /* fall through */
@@ -287,6 +277,19 @@ int main(int argc, char *argv[]) {
 
     target_hostname = argv[0];
 
+
+    if ((ptsc_path != NULL) && (ptsc_conf != NULL)) {
+        int len;
+        INFO("ptsc debug mode\n");
+        len =  strlen(ptsc_path) + strlen(ptsc_conf) + 13;
+        ptsc_command = malloc(len);
+        snprintf(ptsc_command, len, "%s -m -v -c %s", ptsc_path, ptsc_conf);
+        INFO("command: %s\n", ptsc_command);
+    }
+
+    /* default command is to verify */
+    if (command == NONE) command = VERIFY;
+
     /* set the DEBUG level, 1,2,3 */
     if (debug > 2) {
         verbose = DEBUG_FLAG | DEBUG_FSM_FLAG | DEBUG_IFM_FLAG;
@@ -296,6 +299,8 @@ int main(int argc, char *argv[]) {
         verbose = DEBUG_FLAG;
     }
 
+    DEBUG("DEBUG mode (%d)\n", debug);
+
     /* new config */
     conf = newPtsConfig();
     if (conf == NULL) {
@@ -317,8 +322,13 @@ int main(int argc, char *argv[]) {
         /* check dir */
         if (checkDir(dirbuf) != PTS_SUCCESS) {
             char ans;
-            printf("%s is missing. create [Y/n]:", dirbuf);
-            rc = scanf("%[YyNn]", &ans);
+            if (isatty(STDIN_FILENO)) {
+                printf("%s is missing. create [Y/n]:", dirbuf);
+                rc = scanf("%[YyNn]", &ans);
+            } else {
+                rc = 1;
+                ans = 'Y';
+            }
 
             /* new UUID */
             conf->uuid = newOpenptsUuid();
@@ -381,21 +391,11 @@ int main(int argc, char *argv[]) {
             free(conf->hostname);
         }
         conf->hostname = smalloc(target_hostname);
-        conf->port = port;
         target_collector =  getTargetCollector(conf);
         if (target_collector == NULL) {
             /* missing, new target => malloc temp target */
-#if 0
-            target_collector = (OPENPTS_TARGET *) malloc(sizeof(OPENPTS_TARGET));
-            memset(target_collector, 0, sizeof(OPENPTS_TARGET));
-            target_collector->state = 999;  // TODO temp
-            target_collector->target_conf = newPtsConfig();  // TODO free
-            new_target = 1;
-            target_conf = target_collector->target_conf;
-#else
             new_target = 1;
             target_conf = newPtsConfig();
-#endif
         } else {
             /* HIT exist */
             target_conf_dir  = getTargetConfDir(conf);  // HOME/.openpts/UUID
@@ -504,131 +504,41 @@ int main(int argc, char *argv[]) {
         /* check the target dir  */
         if (checkDir(target_conf_dir) == PTS_SUCCESS) {
             initialized++;  // 1->2
-
-            // rc = readTargetConf(conf, target_conf_filename);
-            // if ((rc != PTS_SUCCESS) && (force == 0)) {
-            //     ERROR("%s dir exist, but target.conf is missing. please initialize again with -f option\n",
-            //         target_conf_dir);
-            // }
         }
     }
 
-    /* check the port # */
-    if (port != target_conf->port) {
-        // TODO to be tested
-        if (target_conf->port == 0) {
-            /* no conf? */
-            target_conf->port = port;
-        } else {
-            DEBUG("port=%d != conf->port=%d (used)\n", port, target_conf->port);
-            port = target_conf->port;
+    /* check for an overriding ssh username */
+    if (ssh_username != NULL) {
+        target_conf->ssh_username = strdup(ssh_username);
+        if (target_conf->ssh_username == NULL) {
+            ERROR("No memory");
+            goto free;
         }
+    } else {
+        ssh_username = target_conf->ssh_username;
+    }
+
+    /* check for an overriding ssh port # */
+    if (ssh_port != NULL) {
+        target_conf->ssh_port = smalloc(ssh_port);
+        if (target_conf->ssh_port == NULL) {
+            ERROR("No memory");
+            goto free;
+        }
+    } else {
+        ssh_port = target_conf->ssh_port;
     }
 
     // TODO reset old settings if exist (previous config)
     if (command == INIT) {
         // TODO ("reset conf side\n");
         // target_collector->target_conf->ssh_mode = 0;
-        target_conf->ssh_mode = 0;
         if (target_conf->hostname != NULL) {
             // DEBUG("realloc target_conf->hostname\n"); TODO realloc happen
             free(target_conf->hostname);
         }
         target_conf->hostname = smalloc(target_hostname);
-        target_conf->port = port;
-    }
-
-    /* SSH */
-    if ((ssh == 1) || (target_conf->ssh_mode == OPENPTS_SSH_MODE_ON)) {
-        pid_t pid;
-        int status;
-        char forward[BUF_SIZE];
-        char ssh_target[BUF_SIZE];
-
-        /* SSH mode */
-        target_conf->ssh_mode = OPENPTS_SSH_MODE_ON;  // enable SSH
-
-        /* SSH username */
-        if (ssh_user != NULL) {
-            /* given by cmd option */
-            if (target_conf->ssh_username != NULL) {
-                if (strcmp(target_conf->ssh_username, ssh_user)) {
-                    /* username was changed? */
-                    TODO("name change -> update SSH forword\n");
-                }
-            } else {
-                /* no username in conf, use given username */
-                target_conf->ssh_username = smalloc(ssh_user);
-            }
-        }
-
-        /* SSH local port */
-        if (ssh_port != 0) {
-            /* given by cmd option */
-            if (target_conf->ssh_port != 0) {
-                if (ssh_port != target_conf->ssh_port) {
-                    printf("SSH localport was changed from %d to %d\n", conf->ssh_port, ssh_port);
-                    target_conf->ssh_port = ssh_port;
-                }
-            } else {
-                /* no username in conf, use given username */
-                target_conf->ssh_port = ssh_port;
-            }
-        }
-
-        /* setup local port forward */
-        // ssh -2 -N -f -L 5557:localhost:5556 foo@localhost
-
-        snprintf(forward, BUF_SIZE, "%d:localhost:%d", target_conf->ssh_port, port);
-        snprintf(ssh_target, BUF_SIZE, "%s@%s", target_conf->ssh_username, target_hostname);
-        /* config namespace : target.ssh.* */
-
-        /* ask user name if not defined yet, save to target.conf,  target.ssh.username */
-
-        /* define local port, target.ssh.local.port */
-        DEBUG("target      : %s\n", target_hostname);
-        DEBUG("ssh forward : %s\n", forward);
-        DEBUG("ssh target  : %s\n", ssh_target);
-
-        if (ssh_config == 1) {
-            /* ssh -L */
-            pid = fork();
-            if (pid < 0) {
-                ERROR("\n");
-                rc = -1;
-                goto error;
-            }
-
-            if (pid == 0) {
-                /* child */
-                DEBUG("exec      : /usr/bin/ssh -2 -N -f -L %s %s\n", forward, ssh_target);
-                execl("/usr/bin/ssh", "ssh", "-2", "-N", "-f", "-L", forward, ssh_target,  NULL);
-                /* execl failed */
-                exit(-1);
-            } else {
-                /* parent */
-                waitpid(pid, &status, 0);
-                // TODO ("status = %d\n", status);
-                if (WIFEXITED(status)) {
-                    /* 1 : OK */
-                    // TODO ("Exit status %d\n", WEXITSTATUS(status));
-                } else if (WIFSIGNALED(status)) {
-                    ERROR("Signal status %d\n", WIFSIGNALED(status));
-                    rc = -1;
-                    goto error;
-                } else {
-                    ERROR("Bad exit");
-                    rc = -1;
-                    goto error;
-                }
-            }
-        }
-
-        DEBUG("SSH tunnel mode, port=%d\n", target_conf->ssh_port);
-
-        /* OK, connect to the local port */
-        target_hostname = "localhost";
-        port = target_conf->ssh_port;
+        target_conf->ssh_port = smalloc(ssh_port);
     }
 
     /* new context */
@@ -645,7 +555,7 @@ int main(int argc, char *argv[]) {
         /* get UUID, RMs, AIDE DB */
         DEBUG("enroll with %s  (SSH uses localost)\n", target_hostname);
         DEBUG("conf->config_dir %s\n", conf->config_dir);
-        rc =  enroll(ctx, target_hostname, port, conf->config_dir, force);  // verifier.c
+        rc =  enroll(ctx, target_hostname, ssh_username, ssh_port, conf->config_dir, force);  // verifier.c
         if (rc != 0) {
             fprintf(stderr, "enroll was failed, rc = %d\n", rc);
             printReason(ctx);
@@ -661,7 +571,7 @@ int main(int argc, char *argv[]) {
         // }
 
         DEBUG("conf->config_dir %s\n", conf->config_dir);
-        rc =  verifier(ctx, target_hostname, port, conf->config_dir, 1);  // init
+        rc =  verifier(ctx, target_hostname, ssh_username, ssh_port, conf->config_dir, 1);  // init
         if (rc != OPENPTS_RESULT_VALID) {
             fprintf(stderr, "initial verification was failed, rc = %d\n", rc);
             printReason(ctx);
@@ -679,16 +589,13 @@ int main(int argc, char *argv[]) {
                     printf("manifest[%d]       : %s\n", i, ctx->target_conf->rm_filename[i]);
                 }
             }
-            printf("configulation     : %s\n", ctx->target_conf->config_file);
+            printf("configuration     : %s\n", ctx->target_conf->config_file);
             printf("validation policy : %s\n", ctx->target_conf->policy_filename);
         } else {
             // TODO never happen?
-            printf("configulation     : new target\n");
+            printf("configuration     : new target\n");
         }
 
-
-
-
         // printf("validation policy : %s\n", ctx->conf->policy_filename);
         // printf("aide database     : %s\n", ctx->conf->aide_database_filename);
         // printf("aide ignore list  : %s\n", ctx->conf->aide_ignorelist_filename);
@@ -706,7 +613,7 @@ int main(int argc, char *argv[]) {
         ctx->conf->ima_validation_unknown = 1;
 
         /* vefify*/
-        rc =  verifier(ctx, target_hostname, port, conf->config_dir, 0);  // normal
+        rc =  verifier(ctx, target_hostname, ssh_username, ssh_port, conf->config_dir, 0);  // normal
 
         /* messages */
         // printf("target        : %s\n", argv[0]);
@@ -733,13 +640,10 @@ int main(int argc, char *argv[]) {
                 target_conf->rm_uuid->time->min,
                 target_conf->rm_uuid->time->sec);
 
-            if (target_conf->ssh_mode == 1) {
-                printf("port              : %d (remotehost)\n", target_conf->port);
-                printf("port(ssh)         : %d (localhost)\n", target_conf->ssh_port);
-                printf("username(ssh)     : %s\n", target_conf->ssh_username);
-            } else {
-                printf("port              : %d (localhost)\n", target_conf->port);
-            }
+            printf("username(ssh)     : %s\n",
+                conf->ssh_username ? conf->ssh_username : "default");
+            printf("port(ssh)         : %s\n",
+                conf->ssh_port ? conf->ssh_port : "default");
             printf("policy file       : %s\n", target_conf->policy_filename);
             printf("property file     : %s\n", target_conf->prop_filename);  // TODO ptoperty or prop
         } else {
@@ -764,10 +668,15 @@ int main(int argc, char *argv[]) {
 
 
 #ifdef CONFIG_AUTO_RM_UPDATE
-        if (conf->newrm_exist > 0) {
-            char uuid_str[40]; // 37?
+        // Verify() check the ARU
+        // remote : conf->aru_newrm_uuid
+        // local  : target_conf->newrm_uuid 
+        if ((target_conf != NULL) && (conf->newrm_exist > 0)) {
+            //char uuid_str[40];  // 37?
+            char *uuid_str;
             PTS_DateTime *uuid_time;
             char ans;
+            int same = 0;
 
             if (verbose) {
                 DEBUG("NEWRM_UUID\n");
@@ -778,7 +687,8 @@ int main(int argc, char *argv[]) {
                     printf("NEWRM UUID (local)   : missing\n");
                 }
             }
-            /* Check local newrm */
+
+            /* Check local newrm vs remote newrm */
             if (target_conf->newrm_uuid->uuid != NULL) {
                 if (memcmp(
                         (BYTE*)conf->aru_newrm_uuid,
@@ -786,7 +696,7 @@ int main(int argc, char *argv[]) {
                     /* HIT */
                     printf("---------------------------------------------------------\n");
                     printf("New Manifest UUID : %s ", target_conf->newrm_uuid->str);
-                    printf("(date: %04d-%02d-%02d-%02d:%02d:%02d)\n",
+                    printf("(date: %04d-%02d-%02d-%02d:%02d:%02d) - local\n",
                         target_conf->newrm_uuid->time->year + 1900,
                         target_conf->newrm_uuid->time->mon + 1,
                         target_conf->newrm_uuid->time->mday,
@@ -796,14 +706,17 @@ int main(int argc, char *argv[]) {
                     goto free;
                 } else {
                     /* local is old? */
-                    printf("New reference manifest has been received, but update exist\n");
+                    //printf("New reference manifest has been received, but update exist\n");
+                    same = 1;
                 }
             }
 
             /* msg */
             printf("---------------------------------------------------------\n");
-            encodeBase64((unsigned char *)uuid_str, (unsigned char *)conf->aru_newrm_uuid, 16);
+            //encodeBase64((unsigned char *)uuid_str, (unsigned char *)conf->aru_newrm_uuid, 16);
             uuid_time = getDateTimeOfUuid(conf->aru_newrm_uuid);
+            // base64 printf("New Manifest UUID : %s ", uuid_str);
+            uuid_str = getStringOfUuid(conf->aru_newrm_uuid);
             printf("New Manifest UUID : %s ", uuid_str);
             printf("(date: %04d-%02d-%02d-%02d:%02d:%02d)\n",
                 uuid_time->year + 1900,
@@ -812,25 +725,34 @@ int main(int argc, char *argv[]) {
                 uuid_time->hour,
                 uuid_time->min,
                 uuid_time->sec);
+            free(uuid_str);
 
-            printf("New reference manifest exist. update? [Y/n]\n");
-            rc = scanf("%[YyNn]", &ans);
-
-            DEBUG("conf->config_dir %s\n", conf->config_dir);
-
-            if ((ans == 'Y') || (ans == 'y')) {
-                rc = updateNewRm(ctx, target_hostname, port, conf->config_dir);  // aru.c
-                if (rc == PTS_SUCCESS) {
-                    printf("Save new reference manifest\n");
-                    // TODO UUID
+            if (same == 0) {
+                // printf("New reference manifest exist. update? [Y/n]\n");
+                if (isatty(STDIN_FILENO)) {
+                    printf("New reference manifest exist. update? [Y/n]\n");
+                    rc = scanf("%[YyNn]", &ans);
                 } else {
+                    rc = 1;
+                    ans = update_by_default ? 'Y' : 'N';
+                }
+
+                DEBUG("conf->config_dir %s\n", conf->config_dir);
+
+                if ((ans == 'Y') || (ans == 'y')) {
+                    rc = updateNewRm(ctx, target_hostname, ssh_username, ssh_port, conf->config_dir);  // aru.c
+                    if (rc == PTS_SUCCESS) {
+                        printf("Save new reference manifest\n");
+                        // TODO UUID
+                    } else {
+                    }
+                } else if ((ans == 'N') || (ans == 'n')) {
+                    printf("keep current manifest\n");
+                } else {
+                    printf("Bad answer %c, exit\n", ans);
+                    rc = -1;
+                    goto error;
                 }
-            } else if ((ans == 'N') || (ans == 'n')) {
-                printf("keep current manifest\n");
-            } else {
-                printf("Bad answer %c, exit\n", ans);
-                rc = -1;
-                goto error;
             }
 
             // TODO validate new RM
@@ -858,29 +780,9 @@ int main(int argc, char *argv[]) {
  error:
  free:
     if (target_conf_dir != NULL) free(target_conf_dir);
+
     if (target_conf_filename != NULL) free(target_conf_filename);
 
-    // TODO just use the target?
-    // if ((new_target == 1) && (target_collector  != NULL)) {
-    //    if (target_collector->str_uuid != NULL) free(target_collector->str_uuid);
-    //    if (target_collector->uuid != NULL) free(target_collector->uuid);
-    //    if (target_collector->time != NULL) free(target_collector->time);
-    //    if (target_collector->dir != NULL) free(target_collector->dir);
-    //    if (target_collector->target_conf_filename != NULL) free(target_collector->target_conf_filename);
-    //    if (target_collector->target_conf != NULL) freePtsConfig((OPENPTS_CONFIG *)target_collector->target_conf);
-    // }
-
-    // if (ctx->target_conf != NULL) {
-    //    freePtsConfig(ctx->target_conf);
-    //    ctx->target_conf = NULL;
-    // }
-    // if (ctx->target_conf->config_file != NULL) {
-    //    // TODO somehow valgrind report the leak
-    //    // TODO("ctx->target_conf->config_file != NULL\n");
-    //    free(ctx->target_conf->config_file);
-    //    ctx->target_conf->config_file = NULL;
-    // }
-
     if ((new_target == 1) && (target_conf != NULL)) {
         /* free new target conf */
         freePtsConfig(target_conf);
