@@ -756,7 +756,8 @@ int writeCoreAssertionInfo(xmlTextWriterPtr writer, OPENPTS_CONTEXT * ctx, int l
  */
 int writeRm(OPENPTS_CONTEXT * ctx, const char *file, int level) {
     int rc = 0;
-    int i, j;
+    int i;
+    // int j;
     xmlTextWriterPtr writer;
     xmlBufferPtr buf;
     PTS_UUID *ir_uuid;
@@ -892,7 +893,7 @@ int writeRm(OPENPTS_CONTEXT * ctx, const char *file, int level) {
     /* core:DigestMethod elements */
     DEBUG("writeRm - core:DigestMethod - TBD\n");
 
-    j = 0;
+    // j = 0;
 
     /* core:Values loop */
     DEBUG("writeRm - core:Values- loop\n");
@@ -949,6 +950,13 @@ int writeRm(OPENPTS_CONTEXT * ctx, const char *file, int level) {
         goto free;
     }
 
+    rc = xmlTextWriterFlush(writer);
+    if (rc < 0) {
+        ERROR("writeRm: Error at xmlTextWriterFlush\n");
+        rc = PTS_INTERNAL_ERROR;
+        goto free;
+    }
+
     /* Close all elements */
     rc = xmlTextWriterEndDocument(writer);
     if (rc < 0) {
@@ -1450,7 +1458,8 @@ int makeRmSetDir(OPENPTS_CONFIG *conf) {
 
         if (lstat(buf, &st) == -1) {
             /* Missing conf dir => create */
-            rc = mkdir(buf, S_IRUSR | S_IWUSR | S_IXUSR);
+            rc = mkdir(buf, S_IRUSR | S_IWUSR | S_IXUSR |
+                            S_IRGRP | S_IWGRP | S_IXGRP);
             if (rc != 0) {
                 ERROR("create conf directory, %s was failed\n", buf);
                 rc = PTS_INTERNAL_ERROR;
@@ -1497,7 +1506,8 @@ int makeNewRmSetDir(OPENPTS_CONFIG *conf) {
 
         if (lstat(buf, &st) == -1) {
             /* Missing conf dir => create */
-            rc = mkdir(buf, S_IRUSR | S_IWUSR | S_IXUSR);
+            rc = mkdir(buf, S_IRUSR | S_IWUSR | S_IXUSR |
+                            S_IRGRP | S_IWGRP | S_IXGRP);
             if (rc != 0) {
                 ERROR("create conf directory, %s was failed\n", buf);
                 rc = PTS_INTERNAL_ERROR;
