@@ -26,8 +26,8 @@
  * \brief TSS wrapper
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-08-18
- * cleanup 2011-01-22 SM
  * refactoring 2011-02-15 SM
+ * cleanup 2011-04-26 SM
  *
  * Create Sign Key
  * Create AIK
@@ -37,7 +37,8 @@
  *   uuit_t    uuid/uuid.h       typedef unsigned char uuid_t[16];
  *   TSS_UUID  tss/tss_structs.h 16-bytes struct
  *
- * TODO use quote2
+ * return is TSS_XXX
+ *
  */
 
 #include <stdio.h>
@@ -57,6 +58,7 @@
 #include <openssl/rsa.h>
 #include <openssl/bn.h>
 #include <openssl/objects.h>
+#include <openssl/err.h>
 
 #include <openpts.h>
 
@@ -70,27 +72,31 @@
 /* ONLY for verifier side */
 int printTssKeyList(int ps_type) {
     /* dummy */
-    return 0;
+    return TSS_SUCCESS;
 }
+
 int createTssSignKey(PTS_UUID *uuid, int ps_type, char *filename, int force, int srk_password_mode) {
     /* dummy */
-    return 0;
+    return TSS_SUCCESS;
 }
+
 int deleteTssKey(PTS_UUID *uuid, int ps_type) {
     /* dummy */
-    return 0;
+    return TSS_SUCCESS;
 }
+
 int createAIK() {
     /* dummy */
     TODO("createAIK - TBD\n");
-    return -1;
+    return TSS_FATAL;
 }
+
 int getTssPubKey(
     PTS_UUID *uuid,
     int ps_type, int srk_password_mode,
     int resetdalock, char *filename, int *pubkey_length, BYTE **pubkey) {
     /* dummy */
-    return 0;
+    return TSS_SUCCESS;
 }
 
 int quoteTss(
@@ -106,7 +112,7 @@ int quoteTss(
         /* Output */
         TSS_VALIDATION *validationData) {
     /* dummy */
-    return 0;
+    return TSS_SUCCESS;
 }
 
 int quote2Tss(
@@ -122,7 +128,7 @@ int quote2Tss(
         /* Output */
         TSS_VALIDATION *validationData) {
     /* dummy */
-    return 0;
+    return TSS_SUCCESS;
 }
 
 int getRandom(BYTE *out, int size) {
@@ -147,7 +153,7 @@ int readPcr(int pcr_index, BYTE *pcr) {
 }
 
 #else  // CONFIG_NO_TSS
-/* Collector side */
+/* TSS - Collector side */
 
 BYTE null_srk_auth[1] = {0};  // ""
 BYTE known_srk_auth[20] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -341,7 +347,7 @@ int printTssKeyList(int ps_type) {
     TSS_RESULT result = 0;
     TSS_HCONTEXT hContext;
     UINT32 ulKeyHierarchySize;
-    BYTE *buf;
+    // BYTE *buf;
     TSS_UUID SRK_UUID = TSS_UUID_SRK;
     int i;
     TSS_KM_KEYINFO *info = NULL;
@@ -381,11 +387,11 @@ int printTssKeyList(int ps_type) {
     printf("Key number   : %d\n", ulKeyHierarchySize);
     for (i = 0; i < (int)ulKeyHierarchySize; i++) {
         printf("Key %d\n", i);
-        buf = (BYTE *) & info->versionInfo;
+        // buf = (BYTE *) & info->versionInfo;
         // printhex(" version     : ", buf, 4);
-        buf = (BYTE *) & info->keyUUID;
+        // buf = (BYTE *) & info->keyUUID;
         // printhex(" uuid        : ", buf, 16);
-        buf = (BYTE *) & info->parentKeyUUID;
+        // buf = (BYTE *) & info->parentKeyUUID;
         // printhex(" parents uuid: ", buf, 16);
         info = info + 1;
     }
@@ -423,7 +429,7 @@ int createTssSignKey(PTS_UUID *uuid, int ps_type, char *filename, int force, int
     BYTE *keyBlob;
     int i;
     TSS_UUID tss_uuid;
-    BYTE *buf;
+    // BYTE *buf;
     TSS_FLAG initFlag = TSS_KEY_SIZE_2048 | TSS_KEY_TYPE_SIGNING;
 
     /* Open TSS */
@@ -580,7 +586,7 @@ int createTssSignKey(PTS_UUID *uuid, int ps_type, char *filename, int force, int
                     ERROR("Tspi_Context_RegisterKey failed rc=0x%x\n",
                      result);
                     ERROR("       TSS_E_KEY_ALREADY_REGISTERED\n");
-                    buf = (BYTE *) & uuid;
+                    // buf = (BYTE *) & uuid;
                     // printhex("       uuid=", buf, 16);
                 }
             } else {
@@ -665,7 +671,6 @@ int deleteTssKey(PTS_UUID *uuid, int ps_type) {
  * conf->pubkey_length
  * conf->pubkey
  */
-// int getTssPubKey(PTS_UUID *uuid, int ps_type,  char *filename, OPENPTS_CONFIG *conf) {
 int getTssPubKey(
     PTS_UUID *uuid,
     int ps_type,
@@ -1252,11 +1257,6 @@ int quoteTss(
         validation_data.ulValidationDataLength);
 
     /* version */
-    // validationData->versionInfo.bMajor    = validation_data.versionInfo.bMajor;
-    // validationData->versionInfo.bMinor    = validation_data.versionInfo.bMinor;
-    // validationData->versionInfo.bRevMajor = validation_data.versionInfo.bRevMajor;
-    // validationData->versionInfo.bRevMinor = validation_data.versionInfo.bRevMinor;
-    // get from validationData->rgbData (used by Quote)
     validationData->versionInfo.bMajor    = validationData->rgbData[0];
     validationData->versionInfo.bMinor    = validationData->rgbData[1];
     validationData->versionInfo.bRevMajor = validationData->rgbData[2];
@@ -1313,7 +1313,7 @@ int quoteTss(
     Tspi_Context_CloseObject(hContext, hSRKPolicy);
     Tspi_Context_CloseObject(hContext, hSRK);
     Tspi_Context_CloseObject(hContext, hTPM);
-    free(nonce);
+    // free(nonce);
 
     /* Close TSS/TPM */
   close:
@@ -1699,7 +1699,7 @@ int quote2Tss(
     Tspi_Context_FreeMemory(hContext, NULL);
     Tspi_Context_CloseObject(hContext, hPcrComposite);
     Tspi_Context_CloseObject(hContext, hKey);
-    free(nonce);
+    // free(nonce);
 
     /* Close TSS/TPM */
   close:
@@ -1912,6 +1912,8 @@ int readPcr(int pcr_index, BYTE *pcr) {
  * validate QuoteData
  *
  * - TPM_SS_RSASSAPKCS1v15_SHA1 ONLY
+ * - TPM_Quote
+ * - TPM_Quote2
  *
  *  Return
  *    PTS_SUCCESS
@@ -1935,6 +1937,8 @@ int validateQuoteData(OPENPTS_PCRS *pcrs, TSS_VALIDATION *validationData) {
     BIGNUM *rsa_e = NULL;
     BIGNUM *rsa_n = NULL;
     BYTE exp[4] = {0x00, 0x01, 0x00, 0x01};
+    char *rsa_e_hex = NULL;
+    char *rsa_n_hex = NULL;
 
     /* check */
     if (pcrs == NULL) {
@@ -1999,10 +2003,15 @@ int validateQuoteData(OPENPTS_PCRS *pcrs, TSS_VALIDATION *validationData) {
 
     */
 
-
+    // TODO 2048 bit key only
     pubkey_length = 256;  // TODO
     pubkey = &pcrs->pubkey[28];  // TODO use struct
 
+    // pubkey_length = 257;
+    // pubkey = malloc(pubkey_length);
+    // pubkey[0] = 0;
+    // memcpy(&pubkey[1],&pcrs->pubkey[28], 256);
+
 #if 0
     TODO("\n");
     printHex("message   :", message, message_length, "\n");
@@ -2022,8 +2031,11 @@ int validateQuoteData(OPENPTS_PCRS *pcrs, TSS_VALIDATION *validationData) {
     rsa_n = BN_new();
     BN_bin2bn(pubkey, pubkey_length, rsa_n);
 
-    BN_hex2bn(&(rsa->n), BN_bn2hex(rsa_n));
-    BN_hex2bn(&(rsa->e), BN_bn2hex(rsa_e));
+    rsa_e_hex = BN_bn2hex(rsa_e);
+    rsa_n_hex = BN_bn2hex(rsa_n);
+
+    BN_hex2bn(&(rsa->n), rsa_n_hex);
+    BN_hex2bn(&(rsa->e), rsa_e_hex);
 
     // DEBUG("RSA_verify\n");
     /* RSA verify  1: success, 0:otherwise */
@@ -2039,6 +2051,12 @@ int validateQuoteData(OPENPTS_PCRS *pcrs, TSS_VALIDATION *validationData) {
     RSA_free(rsa);
     BN_free(rsa_e);
     BN_free(rsa_n);
+    OPENSSL_free(rsa_e_hex);
+    OPENSSL_free(rsa_n_hex);
+
+    if (hash != NULL) {
+        free(hash);
+    }
 
     /* DEBUG */
     if (verbose & DEBUG_FLAG) {
@@ -2056,6 +2074,14 @@ int validateQuoteData(OPENPTS_PCRS *pcrs, TSS_VALIDATION *validationData) {
         return PTS_SUCCESS;
     } else {
         /* RSA verify - fail */
+        UINT32 e;  // unsigned long
+        ERR_load_crypto_strings();
+        e = ERR_get_error();
+        ERROR("RSA_verify fail, %s\n", ERR_error_string(e, NULL));
+        ERROR("   %s\n", ERR_lib_error_string(e));
+        ERROR("   %s\n", ERR_func_error_string(e));
+        ERROR("   %s\n", ERR_reason_error_string(e));
+        ERR_free_strings();
         return PTS_VERIFY_FAILED;
     }
 }
@@ -2298,7 +2324,7 @@ int validatePcrCompositeV12(OPENPTS_PCRS *pcrs, TSS_VALIDATION *validationData)
     loc += 4;
     /* PCR values */
     ptr = &buf[loc];
-    for (i = 0; i < 16; i++) {
+    for (i = 0; i < MAX_PCRNUM; i++) {
         if (pcrs->pcr_select[i] == 1) {
             memcpy(ptr, pcrs->pcr[i], 20);
             ptr += 20;
