@@ -26,7 +26,7 @@
  * \brief UUID wrapper (Generic part, OPENPTS_UUID)
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-11-29
- * cleanup 2011-01-20,21 SM
+ * cleanup 2011-04-26 SM
  *
  * Linux uses libuuid
  *
@@ -62,6 +62,8 @@
 
 /**
  * Create new OPENPTS_UUID, no contents
+ *
+ * @return OPENPTS_UUID
  */
 OPENPTS_UUID *newOpenptsUuid() {
     OPENPTS_UUID *uuid;
@@ -77,7 +79,9 @@ OPENPTS_UUID *newOpenptsUuid() {
 }
 
 /**
- * init UUID from file 
+ * init UUID from file
+ *
+ * @return OPENPTS_UUID
  */
 OPENPTS_UUID *newOpenptsUuidFromFile(char * filename) {
     OPENPTS_UUID *uuid;
@@ -88,7 +92,9 @@ OPENPTS_UUID *newOpenptsUuidFromFile(char * filename) {
 
     uuid->filename = smalloc(filename);
     rc = readOpenptsUuidFile(uuid);
-    // TODO check
+    if (rc != PTS_SUCCESS) {
+        ERROR("newOpenptsUuidFromFile() - readOpenptsUuidFile() fail rc=%d\n", rc);
+    }
 
     return uuid;
 }
@@ -120,7 +126,8 @@ void freeOpenptsUuid(OPENPTS_UUID *uuid) {
 }
 
 /**
- *
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  */
 int genOpenptsUuid(OPENPTS_UUID *uuid) {
     /* check */
@@ -161,6 +168,9 @@ int genOpenptsUuid(OPENPTS_UUID *uuid) {
 
 /**
  * read UUID from file(uuid->filename), and fill OPENPTS_UUID
+ *
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  */
 int readOpenptsUuidFile(OPENPTS_UUID *uuid) {
     int rc = PTS_SUCCESS;
@@ -168,8 +178,6 @@ int readOpenptsUuidFile(OPENPTS_UUID *uuid) {
     char line[BUF_SIZE];
     int i;
 
-
-
     /* check */
     if (uuid == NULL) {
         ERROR("\n");
@@ -239,6 +247,8 @@ int readOpenptsUuidFile(OPENPTS_UUID *uuid) {
 
 /**
  *
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  */
 int writeOpenptsUuidFile(OPENPTS_UUID *uuid, int overwrite) {
     FILE *fp;
@@ -291,10 +301,9 @@ int writeOpenptsUuidFile(OPENPTS_UUID *uuid, int overwrite) {
 /**
  * Generate UUID and Write UUID file 
  *
- * Return
- *  PTS_SUCCESS  ok
- *  PTS_DENIED   if UUID file already exist
- *  PTS_INTERNAL_ERROR  else 
+ * @retval PTS_SUCCESS
+ * @retval PTS_DENIED (if UUID file already exist)
+ * @retval PTS_INTERNAL_ERROR
  */
 int writeUuidFile(char *str_uuid, char *filename, int overwrite) {
     FILE *fp;
@@ -329,10 +338,9 @@ int writeUuidFile(char *str_uuid, char *filename, int overwrite) {
  * TODO deprecated use readOpenptsUuidFile
  * Read UUID file
  * 
- * Return
- *  PTS_SUCCESS        ok
- *  PTS_DENIED         missing uuiod file
- *  PTS_INTERNAL_ERROR
+ * @retval PTS_SUCCESS
+ * @retval PTS_DENIED (missing uuid file)
+ * @retval PTS_INTERNAL_ERROR
  *
  */
 int readUuidFile(char *filename, char **str_uuid, PTS_UUID **uuid) {
@@ -389,9 +397,15 @@ int readUuidFile(char *filename, char **str_uuid, PTS_UUID **uuid) {
  * selectUuidDir
  *
  * select UUID dir, e.g. 12877946-0682-11e0-b442-001f160c9c28
+ *
+ * @retval 0
+ * @retval 1 - hit
  */
 static int selectUuidDir(const struct dirent *entry) {
     int len;
+#ifndef __linux__
+    struct stat buffer;
+#endif
 
     /* skip . .. dirs */
     if (0 == strcmp(".", entry->d_name)) return 0;
@@ -406,14 +420,20 @@ static int selectUuidDir(const struct dirent *entry) {
 
     /* Dir HIT */
     // TODO check the format
+#ifndef __linux__
+    if (0 != stat(entry->d_name, &buffer)) return 0;
+    if (S_ISDIR(buffer.st_mode)) return 1;
+#else
     if (entry->d_type == DT_DIR) return 1;
+#endif
 
     return 0;
 }
 
 /**
- *  time1 > time2    1
- *  time1 <= time2  = 0
+ *
+ *  @retval 0 - time1 <= time2
+ *  @retval 1 - time1 > time2
  */
 int cmpDateTime(PTS_DateTime *time1, PTS_DateTime *time2) {
     uint64_t t1 = 0;
@@ -509,6 +529,9 @@ void printRmList(OPENPTS_CONFIG *conf, char *indent) {
  *
  *   conf->current_rm_uuid
  *   conf->next_rm_uuid
+ *
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  */
 int getRmList(OPENPTS_CONFIG *conf, char * config_dir) {
     int cnt = 0;
@@ -672,13 +695,14 @@ int rmRmsetDir(char * dir) {
 
 /**
  * Purge RMs 
+ *
+ * @retval PTS_SUCCESS
+ * @retval PTS_OS_ERROR
  */
 int purgeRenewedRm(OPENPTS_CONFIG *conf) {
     int cnt;
     int state;
-    char * str_uuid;
     int num = 0;
-    PTS_DateTime *time;
     OPENPTS_RMSET *rmset;
     int rc;
     int rc2 = PTS_SUCCESS;
@@ -688,8 +712,8 @@ int purgeRenewedRm(OPENPTS_CONFIG *conf) {
     /* scan  */
     for (cnt = 0; cnt < num; cnt++) {
         rmset = &conf->rmsets->rmset[cnt];
-        str_uuid = rmset->str_uuid;
-        time = rmset->time;
+        // str_uuid = rmset->str_uuid;
+        // time = rmset->time;
         state = rmset->state;
 
         if (state == OPENPTS_RM_STATE_TRASH) {
@@ -708,6 +732,8 @@ int purgeRenewedRm(OPENPTS_CONFIG *conf) {
 /**
  * list/get target list at HOME/.openpts
  *
+ * @retval PTS_SUCCESS
+ * @retval PTS_INTERNAL_ERROR
  */
 int getTargetList(OPENPTS_CONFIG *conf, char * config_dir) {
     int cnt = 0;
@@ -787,7 +813,7 @@ int getTargetList(OPENPTS_CONFIG *conf, char * config_dir) {
 /**
  *  get the target in target_list which hostname is conf->hostname (given)
  *
- * TODO change the arg
+ * @return dir_string
  */
 char *getTargetConfDir(OPENPTS_CONFIG *conf) {
     char *dir = NULL;
@@ -820,7 +846,7 @@ char *getTargetConfDir(OPENPTS_CONFIG *conf) {
 /**
  *  get the target in target_list which hostname is conf->hostname (given)
  *
- * TODO change the arg
+ * @return OPENPTS_TARGET
  */
 OPENPTS_TARGET *getTargetCollector(OPENPTS_CONFIG *conf) {
     int cnt;
@@ -852,13 +878,16 @@ OPENPTS_TARGET *getTargetCollector(OPENPTS_CONFIG *conf) {
     return NULL;
 }
 
+/**
+ *
+ */
 void printTargetList(OPENPTS_CONFIG *conf, char *indent) {
     int cnt;
     PTS_DateTime *time;
-    int state;
+    // int state;
     OPENPTS_TARGET *target;
     OPENPTS_CONFIG *target_conf;
-    char * str_uuid = NULL;
+    char * str_uuid = "N/A";
     int num = 0;
 
     num = conf->target_list->target_num;
@@ -874,7 +903,7 @@ void printTargetList(OPENPTS_CONFIG *conf, char *indent) {
         target_conf = (OPENPTS_CONFIG *) target->target_conf;
 
         time = target->time;
-        state = target->state;
+        // state = target->state;
 
         if (target_conf != NULL) {
             if (target_conf->uuid != NULL) {
@@ -882,35 +911,19 @@ void printTargetList(OPENPTS_CONFIG *conf, char *indent) {
                     str_uuid = target_conf->uuid->str;
                 }
             }
-            if (target_conf->ssh_port > 0) {
-                printf("%s %3d %s %04d-%02d-%02d-%02d:%02d:%02d %4d %4d     %s@%s\n",
-                    indent,
-                    cnt,
-                    str_uuid,
-                    time->year + 1900,
-                    time->mon + 1,
-                    time->mday,
-                    time->hour,
-                    time->min,
-                    time->sec,
-                    target_conf->port,
-                    target_conf->ssh_port,
-                    target_conf->ssh_username,
-                    target_conf->hostname);
-            } else {
-                printf("%s %3d %s %04d-%02d-%02d-%02d:%02d:%02d %4d  --      %s\n",
-                    indent,
-                    cnt,
-                    str_uuid,
-                    time->year + 1900,
-                    time->mon + 1,
-                    time->mday,
-                    time->hour,
-                    time->min,
-                    time->sec,
-                    target_conf->port,
-                    target_conf->hostname);
-            }
+            printf("%s %3d %s %04d-%02d-%02d-%02d:%02d:%02d %s@%s:%s\n",
+                indent,
+                cnt,
+                str_uuid,
+                time->year + 1900,
+                time->mon + 1,
+                time->mday,
+                time->hour,
+                time->min,
+                time->sec,
+                target_conf->ssh_username ? target_conf->ssh_username : "default",
+                target_conf->hostname,
+                target_conf->ssh_port ? target_conf->ssh_port : "default");
         } else {
             printf("--\n");
         }
