@@ -26,7 +26,7 @@
  * \brief TCG IF-M Verifier
  * @author Seiji Munetoh <munetoh@users.sourceforge.jp>
  * @date 2010-04-06
- * cleanup 2011-01-21
+ * cleanup 2011-04-26 SM
  *
  */
 
@@ -42,6 +42,7 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <netinet/in.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -56,30 +57,30 @@
  * get target RMs
  * else?
  *
- * @param ctx
- * @param host
- * @param port
- * @param ir_filename
- *
- *
  * Function Test
  *
  *   file         test
  *   ----------------------------------
  *   
  */
-int enroll(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int force) {
+int enroll(
+    OPENPTS_CONTEXT *ctx,
+    char *host,
+    char *ssh_username,
+    char *ssh_port,
+    char *conf_dir,
+    int force) {
     int sock;
     int rc = PTS_SUCCESS;
-    struct hostent *server_ip;
-    struct sockaddr_in server_addr;
     PTS_IF_M_Attribute *read_tlv = NULL;
     char *str_verifier_uuid = NULL;
     char buf[BUF_SIZE];
     int i;
     struct stat st;
+    pid_t ssh_pid;
+    int ssh_status;
 
-    OPENPTS_CONFIG *conf;
+    // OPENPTS_CONFIG *conf;
     OPENPTS_CONFIG *target_conf;
     PTS_Byte *value;
     OPENPTS_IF_M_Capability *cap;
@@ -93,7 +94,6 @@ int enroll(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int forc
         ERROR("ctx->conf\n");
         return PTS_INTERNAL_ERROR;
     }
-    conf = ctx->conf;
 
     if (ctx->target_conf == NULL) {
         TODO("enroll() - ctx->target_conf must be set before\n");
@@ -108,44 +108,29 @@ int enroll(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int forc
         target_conf->rm_uuid = newOpenptsUuid();
     }
 
+    /* connect to the target collector */
+    ssh_pid = ssh_connect(host,
+                          ssh_username,
+                          ssh_port,
+                          NULL,
+                          &sock);
 
-    /* setup socket connection */
-    sock = socket(AF_INET, SOCK_STREAM, 0);
-    if (sock < 0) {
-        ERROR("enroll - cl:socket");
-        return PTS_OS_ERROR;
-    }
-
-    server_ip = gethostbyname(host);
-    if (server_ip == NULL) {
-        ERROR("enroll - server:gethostbyname");
+    if (ssh_pid == -1) {
+        ERROR("connection to %s failed.\n", host);
         rc = PTS_OS_ERROR;
-        goto close;
-    }
-
-    memset((char *) &server_addr, 0, sizeof(server_addr));
-    server_addr.sin_family = AF_INET;
-    server_addr.sin_port = htons(port);
-    memcpy((char *) &server_addr.sin_addr,
-           (char *) server_ip->h_addr,
-           server_ip->h_length);
-
-    rc = connect(
-            sock,
-            (struct sockaddr *)&server_addr,
-            sizeof(server_addr));
-
-    if (rc == -1) {
-        ERROR("connection to %s:%d was failed.\n", host, port);
-        rc = PTS_OS_ERROR;
-        goto close;
+        goto out;
     }
 
     /* V->C capability (hello) */
-    rc = writePtsTlvToSock(ctx, sock, OPENPTS_CAPABILITIES);
+    rc = writePtsTlv(ctx, sock, OPENPTS_CAPABILITIES);
 
     /* C->V capability (hello) */
-    read_tlv = readPtsTlvFromSock(sock);
+    read_tlv = readPtsTlv(sock);
+    if (read_tlv == NULL) {
+        ERROR("Can not get the message from collector\n");
+        rc = PTS_INTERNAL_ERROR;
+        goto close;
+    }
     if (read_tlv->type != OPENPTS_CAPABILITIES) {
         ERROR("\n");
         rc = PTS_INTERNAL_ERROR;
@@ -263,7 +248,7 @@ int enroll(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int forc
 
 
     /* V->C template RIMM req  */
-    rc = writePtsTlvToSock(ctx, sock, REQUEST_RIMM_SET);
+    rc = writePtsTlv(ctx, sock, REQUEST_RIMM_SET);
 
     if (rc != 0) {
         ERROR("template RIMM req was failed\n");
@@ -272,7 +257,7 @@ int enroll(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int forc
     }
 
     /* C->V template RIMM (RIMM embedded to CTX) */
-    read_tlv = readPtsTlvFromSock(sock);
+    read_tlv = readPtsTlv(sock);
     if (read_tlv->type != RIMM_SET) {
         ERROR("");
         rc = PTS_INTERNAL_ERROR;
@@ -358,7 +343,7 @@ int enroll(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int forc
 #ifdef CONFIG_AIDE
     // TODO(munetoh) capability defile validation mode of collector
     /* V->C AIDE_DATABASE req  */
-    rc = writePtsTlvToSock(ctx, sock, REQUEST_AIDE_DATABASE);
+    rc = writePtsTlv(ctx, sock, REQUEST_AIDE_DATABASE);
 
     if (rc != 0) {
         ERROR("template RIMM req was failed\n");
@@ -367,39 +352,48 @@ int enroll(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int forc
     }
 
     /* C->V AIDE DATABASE */
-    read_tlv = readPtsTlvFromSock(sock);
+    read_tlv = readPtsTlv(sock);
     if (read_tlv->type != AIDE_DATABASE) {
-        ERROR("");
-        rc = PTS_INTERNAL_ERROR;
-        goto close;
-    }
-
-    if (read_tlv->length > 0) {
-        /* AIDE_DATABASE -> CTX */
-        DEBUG("AIDE_DATABASE size     : %d\n", read_tlv->length);
-
-        rc = saveToFile(target_conf->aide_database_filename, read_tlv->length, read_tlv->value);
-        if (rc < 0) {
-            ERROR("enroll - save AIDE DB failed\n");
+        if (read_tlv->type == OPENPTS_ERROR) {
+            // TODO check msg?
+            /* AIDE DB is missing */
+            target_conf->ima_validation_mode = OPENPTS_VALIDATION_MODE_NONE;
+            DEBUG("enroll - AIDE DB is missing. do not validate IMA's IMLs\n");
+        } else {
+            ERROR("");
             rc = PTS_INTERNAL_ERROR;
             goto close;
         }
+    } else {
+        // Got AIDE DB?
+        if (read_tlv->length > 0) {
+            /* AIDE_DATABASE -> CTX */
+            DEBUG("AIDE_DATABASE size     : %d\n", read_tlv->length);
+
+            rc = saveToFile(target_conf->aide_database_filename, read_tlv->length, read_tlv->value);
+            if (rc < 0) {
+                ERROR("enroll - save AIDE DB failed\n");
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
+            }
 
 #ifdef CONFIG_SQLITE
-        DEBUG("conv to sqlite %s\n", target_conf->aide_sqlite_filename);
-        rc = convertAideDbfileToSQLiteDbFile(ctx->conf->aide_database_filename, ctx->conf->aide_sqlite_filename);
-        if (rc != PTS_SUCCESS) {
-            ERROR("enroll - convert AIDE DB to SQLiteDB was failed\n");
-            rc = PTS_INTERNAL_ERROR;
-            goto close;
-        }
+            DEBUG("conv to sqlite %s\n", target_conf->aide_sqlite_filename);
+            rc = convertAideDbfileToSQLiteDbFile(
+                    ctx->conf->aide_database_filename,
+                    ctx->conf->aide_sqlite_filename);
+            if (rc != PTS_SUCCESS) {
+                ERROR("enroll - convert AIDE DB to SQLiteDB was failed\n");
+                rc = PTS_INTERNAL_ERROR;
+                goto close;
+            }
 #endif
-
-        target_conf->ima_validation_mode = OPENPTS_VALIDATION_MODE_AIDE;
-    } else {
-        /* no AIDE DB */
-        target_conf->ima_validation_mode = OPENPTS_VALIDATION_MODE_NONE;
-        DEBUG("enroll - AIDE DB is missing. do not validate IMA's IMLs\n");
+            target_conf->ima_validation_mode = OPENPTS_VALIDATION_MODE_AIDE;
+        } else {
+            /* no AIDE DB */
+            target_conf->ima_validation_mode = OPENPTS_VALIDATION_MODE_NONE;
+            DEBUG("enroll - AIDE DB is missing. do not validate IMA's IMLs\n");
+        }
     }
 
     /* free */
@@ -410,7 +404,7 @@ int enroll(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int forc
 
 
     /* V->C TPM PUBKEY req */
-    rc = writePtsTlvToSock(ctx, sock, REQUEST_TPM_PUBKEY);
+    rc = writePtsTlv(ctx, sock, REQUEST_TPM_PUBKEY);
 
     if (rc != 0) {
         ERROR("TPM PUBKEY req was failed\n");
@@ -419,9 +413,12 @@ int enroll(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int forc
     }
 
     /* C->V TPM PUBKEY */
-    read_tlv = readPtsTlvFromSock(sock);
-    if (read_tlv->type != TPM_PUBKEY) {
-        ERROR("");
+    read_tlv = readPtsTlv(sock);
+    if (read_tlv->type == OPENPTS_ERROR) {
+        // TODO Ignore now
+        TODO("Target did not have TPM_PUBKEY");
+    } else if (read_tlv->type != TPM_PUBKEY) {
+        ERROR("read_tlv->type != TPM_PUBKEY, but %d", read_tlv->type);
         rc = PTS_INTERNAL_ERROR;
         goto close;
     }
@@ -459,7 +456,9 @@ int enroll(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int forc
 
   close:
     close(sock);
+    waitpid(ssh_pid, &ssh_status, 0);
 
+  out:
     /* free */
     if (str_verifier_uuid != NULL) free(str_verifier_uuid);
 
@@ -475,25 +474,25 @@ int enroll(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int forc
  *
  * get target NEW RMs
  *
- * @param ctx
- * @param host
- * @param port
- * @param conf_dir
- *
- *
  * Function Test
  *
  *   file         test
  *   ----------------------------------
  *   
  */
-int updateRm(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir) {
+int updateRm(
+    OPENPTS_CONTEXT *ctx,
+    char *host,
+    char *ssh_username,
+    char *ssh_port,
+    char *conf_dir) {
     int sock;
     int rc;
-    struct hostent *server_ip;
-    struct sockaddr_in server_addr;
     PTS_IF_M_Attribute *read_tlv;
     char *rm_filename[MAX_RM_NUM];
+    pid_t ssh_pid;
+    int ssh_status;
+
     PTS_UUID *collector_uuid;
     char *str_collector_uuid;
     PTS_UUID *rm_uuid;
@@ -517,45 +516,28 @@ int updateRm(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir) {
         return PTS_INTERNAL_ERROR;
     }
 
-    /* setup socket connection */
-    sock = socket(AF_INET, SOCK_STREAM, 0);
-    if (sock < 0) {
-        ERROR("enroll - cl:socket");
-        goto error1;
-    }
+    /* connect to the target collector */
+    ssh_pid = ssh_connect(host,
+                          ssh_username,
+                          ssh_port,
+                          NULL,
+                          &sock);
 
-    server_ip = gethostbyname(host);
-    if (server_ip == NULL) {
-        ERROR("enroll - server:gethostbyname");
-        goto error2;
-    }
-
-    memset((char *) &server_addr, 0, sizeof(server_addr));
-    server_addr.sin_family = AF_INET;
-    server_addr.sin_port = htons(port);
-    memcpy((char *) &server_addr.sin_addr,
-           (char *) server_ip->h_addr,
-           server_ip->h_length);
-
-    rc = connect(
-            sock,
-            (struct sockaddr *)&server_addr,
-            sizeof(server_addr));
-
-    if (rc == -1) {
-        ERROR("connection to %s was failed.\n", host);
-        goto error2;
+    if (ssh_pid == -1) {
+        ERROR("connection to %s failed.\n", host);
+        rc = PTS_OS_ERROR;
+        goto out;
     }
 
     /* V->C capability (hello) */
-    rc = writePtsTlvToSock(ctx, sock, OPENPTS_CAPABILITIES);
+    rc = writePtsTlv(ctx, sock, OPENPTS_CAPABILITIES);
 
     /* C->V capability (hello) */
-    read_tlv = readPtsTlvFromSock(sock);
-    if (read_tlv->type != OPENPTS_CAPABILITIES) goto error2;
+    read_tlv = readPtsTlv(sock);
+    if (read_tlv->type != OPENPTS_CAPABILITIES) goto error;
     if (read_tlv->length != sizeof(OPENPTS_IF_M_Capability)) {  // TODO set name
         ERROR("UUID length = %d != 36\n", read_tlv->length);
-        goto error2;
+        goto error;
     }
     cap = (OPENPTS_IF_M_Capability *) read_tlv->value;
 
@@ -580,7 +562,7 @@ int updateRm(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir) {
         (str_rm_uuid == NULL) ||
         (str_verifier_uuid == NULL)) {
         ERROR("no memory\n");
-        goto error2;
+        goto error;
     }
 
     DEBUG("Verifier  UUID    %s\n", str_verifier_uuid);
@@ -595,11 +577,11 @@ int updateRm(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir) {
     if (rc != PTS_SUCCESS) {
         /* unknwon collector */
         ERROR("Unknown collector, UUID= %s\n", str_collector_uuid);
-        addReason(ctx, "Missing collector configulation");
+        addReason(ctx, "Missing collector configuration");
         addReason(ctx, "Collector hostname = %s", host);
         addReason(ctx, "Collector UUID = %s", str_collector_uuid);
         rc = PTS_NOT_INITIALIZED;
-        goto close;
+        goto error;
     }
     rc = checkDir(rm_dir);
     if (rc == PTS_SUCCESS) {
@@ -610,7 +592,7 @@ int updateRm(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir) {
         addReason(ctx, "Collector UUID = %s", str_collector_uuid);
         addReason(ctx, "Collector RM UUID = %s", str_rm_uuid);
         rc = PTS_FATAL;
-        goto close;
+        goto error;
     }
 
     /* create new RM dir */
@@ -619,7 +601,7 @@ int updateRm(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir) {
         /* unknwon collector */
         ERROR("Create RM dir failed, %s\n", rm_dir);
         rc = PTS_INTERNAL_ERROR;
-        goto close;
+        goto error;
     }
 
     conf->property_filename = getFullpathName(collector_dir, "vr.properties");
@@ -642,18 +624,18 @@ int updateRm(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir) {
     /* get the Reference Manifest from target(collector) */
 
     /* V->C template RIMM req  */
-    rc = writePtsTlvToSock(ctx, sock, REQUEST_RIMM_SET);
+    rc = writePtsTlv(ctx, sock, REQUEST_RIMM_SET);
 
     if (rc != 0) {
         ERROR("template RIMM req was failed\n");
-        goto error2;
+        goto error;
     }
 
     /* C->V template RIMM (RIMM embedded to CTX) */
-    read_tlv = readPtsTlvFromSock(sock);
+    read_tlv = readPtsTlv(sock);
     if (read_tlv->type != RIMM_SET) {
         ERROR("");
-        goto error2;
+        goto error;
     }
 
     /* RIMM -> CTX */
@@ -685,8 +667,8 @@ int updateRm(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir) {
 
             rc = saveToFile(rm_filename[i], len, read_tlv->value);
             if (rc < 0) {
-                ERROR("enroll - save RM[%d], %s failed\n", i, rm_filename[i]);
-                goto error2;
+                ERROR("engage - save RM[%d], %s failed\n", i, rm_filename[i]);
+                goto error;
             }
             ctx->conf->rm_filename[i] = smalloc(rm_filename[i]);
 
@@ -704,15 +686,15 @@ int updateRm(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir) {
     // TODO need to updade?
     // writeTargetConf(ctx->conf, collector_uuid, target_conf_filename);  // ctx.c
 
-    close(sock);
-    return 0;  // OK
+    rc = PTS_SUCCESS;
 
-  close:
-  error2:
+  error:
     close(sock);
-  error1:
+    waitpid(ssh_pid, &ssh_status, 0);
+
+  out:
     // DEBUG("error at verifier\n");
-    return -1;
+    return rc;
 }
 
 
@@ -874,7 +856,7 @@ int  writeAideIgnoreList(OPENPTS_CONTEXT *ctx, char *filename) {
  * @param ctx
  * @param host
  * @param port
- * @param conf_dir - base dir of collector configulation
+ * @param conf_dir - base dir of collector configuration
  * @param mode   0:normal  1:sync (update policy, ignorelist)
  *     0  OPENPTS_VERIFY_MODE
  *     1  OPENPTS_UPDATE_MODE -- note) do not update the RM, use updateRm()
@@ -891,13 +873,20 @@ int  writeAideIgnoreList(OPENPTS_CONTEXT *ctx, char *filename) {
  *   ----------------------------------
  *   check_ifm.c  test_ifm
  */
-int verifier(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int mode) {
+int verifier(
+    OPENPTS_CONTEXT *ctx,
+    char *host,
+    char *ssh_username,
+    char *ssh_port,
+    char *conf_dir,
+    int mode) {
     int rc = PTS_SUCCESS;
     int result = OPENPTS_RESULT_VALID;
     /* sock */
     int sock;
-    struct hostent *server_ip;
-    struct sockaddr_in server_addr;
+    pid_t ssh_pid;
+    int ssh_status;
+
     /* TLV/PTS */
     PTS_IF_M_Attribute *read_tlv = NULL;
     OPENPTS_UUID *rm_uuid = NULL;  // received from collector
@@ -930,40 +919,19 @@ int verifier(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int mo
     }
     target_conf = ctx->target_conf;
 
-    /* open socket */
-    sock = socket(AF_INET, SOCK_STREAM, 0);
-    if (sock < 0) {
-        ERROR("verifier() - sock fail\n");
-        addReason(ctx, "connection was faild");
-        rc = PTS_OS_ERROR;
-        return rc;
-    }
 
-    server_ip = gethostbyname(host);
-    if (server_ip == NULL) {
-        ERROR("verifier() - gethostbyname failed\n");
-        addReason(ctx, "connection was faild");
-        rc = PTS_OS_ERROR;
-        goto close;
-    }
+    /* connect to the target collector */
+    ssh_pid = ssh_connect(host,
+                          ssh_username,
+                          ssh_port,
+                          NULL,
+                          &sock);
 
-    memset((char *) &server_addr, 0, sizeof(server_addr));
-    server_addr.sin_family = AF_INET;
-    server_addr.sin_port = htons(port);
-    memcpy((char *) &server_addr.sin_addr,
-           (char *) server_ip->h_addr,
-           server_ip->h_length);
-
-    rc = connect(
-            sock,
-            (struct sockaddr *)&server_addr,
-            sizeof(server_addr));
-    if (rc == -1) {
-        // ERROR("connection failed (server = %s, port = %d)\n", host, port);
-        fprintf(stderr, "connection failed (server = %s, port = %d)\n", host, port);
-        addReason(ctx, "connection failed (server = %s, port = %d)\n", host, port);
+    if (ssh_pid == -1) {
+        ERROR("connection failed (server = %s)\n", host);
+        addReason(ctx, "connection failed (server = %s)\n", host);
         rc = PTS_OS_ERROR;
-        goto close;
+        goto out;
     }
 
 
@@ -971,12 +939,18 @@ int verifier(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int mo
     /* IF-M start */
 
     /* V->C capability (hello) */
-    rc = writePtsTlvToSock(ctx, sock, OPENPTS_CAPABILITIES);
+    rc = writePtsTlv(ctx, sock, OPENPTS_CAPABILITIES);
 
     /* C->V capability (hello) */
-    read_tlv = readPtsTlvFromSock(sock);
-    if (read_tlv->type != OPENPTS_CAPABILITIES) goto close;
-    if (read_tlv->length != sizeof(OPENPTS_IF_M_Capability)) {  // TODO PTS_CAPABILITIES_SIZE
+    read_tlv = readPtsTlv(sock);
+    if (read_tlv == NULL) {
+        ERROR("can't connect to target, %s\n", host);
+        rc = PTS_INTERNAL_ERROR;
+        goto close;
+    } else if (read_tlv->type != OPENPTS_CAPABILITIES) {
+        goto close;
+    } else if (read_tlv->length != sizeof(OPENPTS_IF_M_Capability)) {
+        // TODO PTS_CAPABILITIES_SIZE
         ERROR("UUID length = %d != 36\n", read_tlv->length);
         rc = PTS_INTERNAL_ERROR;
         goto close;
@@ -1044,7 +1018,7 @@ int verifier(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int mo
         /* unknwon collector */
         ERROR("verifier() - Unknown collector, UUID= %s dir= %s, rc=%d\n",
             target_conf->uuid->str, collector_dir, rc);
-        addReason(ctx, "Missing collector configulation");
+        addReason(ctx, "Missing collector configuration");
         addReason(ctx, "Collector hostname = %s", host);
         addReason(ctx, "Collector UUID = %s", target_conf->uuid->str);
         rc = PTS_NOT_INITIALIZED;
@@ -1080,6 +1054,11 @@ int verifier(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int mo
 
     /* check and setup the RM */
     /* stored RM UUID */
+    if (target_conf->rm_uuid == NULL) {
+        ERROR("target_conf->rm_uuid == NULL\n");
+        rc = PTS_INTERNAL_ERROR;
+        goto close;
+    }
     rc = readOpenptsUuidFile(target_conf->rm_uuid);
     if (rc != PTS_SUCCESS) {
         ERROR("read RM UUID file %s was failed, initialize ptscd first\n", target_conf->rm_uuid->filename);
@@ -1215,14 +1194,20 @@ int verifier(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int mo
     ctx->nonce->req->dh_group_set = DH_GROUP_2;
 
     /*   send req   */
-    rc = writePtsTlvToSock(ctx, sock, DH_NONCE_PARAMETERS_REQUEST);
+    rc = writePtsTlv(ctx, sock, DH_NONCE_PARAMETERS_REQUEST);
 
     /* C->V  D-H nonce param res ---------------------------------- */
 
     freePtsTlv(read_tlv);
 
-    read_tlv = readPtsTlvFromSock(sock);
-    if (read_tlv->type != DH_NONCE_PARAMETORS_RESPONSE) goto close;
+    read_tlv = readPtsTlv(sock);
+    if (read_tlv == NULL) {
+        ERROR("[IF-M] DH_NONCE_PARAMETERS_REQUEST was failed, check the collector");
+        rc = PTS_INTERNAL_ERROR;
+        goto close;
+    } else if (read_tlv->type != DH_NONCE_PARAMETORS_RESPONSE) {
+        goto close;
+    }
     // DEBUG("new read_tlv %p\n",read_tlv);
 
     /* res -> fin */
@@ -1258,15 +1243,19 @@ int verifier(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int mo
     }
 
     /* V->C D-H nonce finish  --------------------------------------------- */
-    rc = writePtsTlvToSock(ctx, sock, DH_NONCE_FINISH);
+    rc = writePtsTlv(ctx, sock, DH_NONCE_FINISH);
 
     /* V->C IR req -------------------------------------------------------- */
-    rc = writePtsTlvToSock(ctx, sock, REQUEST_INTEGRITY_REPORT);
+    rc = writePtsTlv(ctx, sock, REQUEST_INTEGRITY_REPORT);
 
     /* C->V IR ------------------------------------------------------------ */
     freePtsTlv(read_tlv);
-    read_tlv = readPtsTlvFromSock(sock);
-    if (read_tlv->type != INTEGRITY_REPORT) {
+    read_tlv = readPtsTlv(sock);
+    if (read_tlv == NULL) {
+        ERROR("REQUEST_INTEGRITY_REPORT was failed, check the collector");
+        rc = PTS_INTERNAL_ERROR;
+        goto close;
+    } else if (read_tlv->type != INTEGRITY_REPORT) {
         rc = PTS_INTERNAL_ERROR;
         goto close;
     }
@@ -1340,7 +1329,7 @@ int verifier(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int mo
     }
 
     /* V->C VR */
-    rc = writePtsTlvToSock(ctx, sock, VERIFICATION_RESULT);
+    rc = writePtsTlv(ctx, sock, VERIFICATION_RESULT);
     if (rc < 0) {
         rc = PTS_INTERNAL_ERROR;
         goto close;
@@ -1372,7 +1361,9 @@ int verifier(OPENPTS_CONTEXT *ctx, char * host, int port, char *conf_dir, int mo
   close:
     /* close socket */
     close(sock);
+    waitpid(ssh_pid, &ssh_status, 0);
 
+  out:
     /* free */
     if (read_tlv != NULL) freePtsTlv(read_tlv);
     if (str_verifier_uuid != NULL) free(str_verifier_uuid);
